// 1.11.4
(
  function($)
  {
    var commands =
      [
        {
          action: download,
          filter: {resourceTypes: ['document', '!principal']},
          label: 'label-download',
          name :'download',
          privileges: ['read']
        },
        {
          action: editUserCommand,
          filter: {action: filterEditUserCommand, resourceTypes: ['principal']},
          label: 'label-edit-user',
          name :'edit-user',
          privileges: ['read', 'write']
        },
        {
          action: editDocument,
          filter:
            {
              action: filterEditDocumentCommand, resourceTypes: ['document']},
          label: 'label-edit-document',
          name :'edit-document',
          privileges: ['read', 'write']
        },
        {
          action: copy,
          filter: {action: filterCopyCutCommand},
          label: 'label-copy',
          name :'copy',
          privileges: ['read']
        },
        {
          action: cut,
          filter: {action: filterCopyCutCommand},
          label: 'label-cut',
          name: 'cut',
          parentPrivileges: ['unbind']
        },
        {
          action: deleteUrl,
          filter:
            {
              action: filterDeleteCommand,
              neededMethods: ['DELETE']
            },
          label: 'label-delete',
          name: 'delete',
          parentPrivileges: ['unbind']
        },
        {
          action: deleteAll,
          filter:
            {
              action: filterDeleteAllCommand,
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-delete-all',
          name: 'delete-all',
          privileges: ['unbind']
        },
        {
          action: newDocumentAction,
          filter:
            {
              action: filterNewDocumentCommand,
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-new-document',
          name: 'new-document',
          privileges: ['bind']
        },
        {
          action: mkcol,
          filter:
            {
              action: filterMkcolCommand,
              neededMethods: ['MKCOL'],
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-mkcol',
          name: 'mkcol',
          privileges: ['bind']
        },
        {
          action: mkcol,
          filter:
            {
              action: filterNewGroupCommand,
              neededMethods: ['MKCOL']
            },
          label: 'label-new-group',
          name: 'new-group',
          privileges: ['bind']
        },
        {
          action: expandAction,
          filter:
            {
              action: filterExpandCommand,
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-expand',
          name: 'expand',
          privileges: ['read']
        },
        {
          action: createUser,
          filter: {action: filterNewUserCommand},
          label: 'label-new-user',
          name: 'new-user',
          privileges: ['bind']
        },
        {
          action: paste,
          filter:
            {
              action: filterPasteDocumentCommand,
              resourceTypes: ['activity', 'document', 'view']
            },
          label: 'label-paste',
          name: 'paste-document',
          privileges: ['write-content']
        },
        {
          action: paste,
          filter:
            {action: filterPasteFolderCommand, resourceTypes: ['collection']},
          label: 'label-paste',
          name: 'paste-folder',
          privileges: ['bind']
        },
        {
          action: rename,
          filter:
            {action: filterRenameCommand, neededMethods: ['MOVE']},
          label: 'label-rename',
          name: 'rename',
          parentPrivileges: ['bind', 'unbind']
        },
        {
          action: uploadAction,
          filter:
            {
              action: filterUpdateDocumentCommand,
              neededMethods: ['PUT'],
              resourceTypes: ['document']
            },
          label: 'label-upload-document',
          name: 'upload-document',
          privileges: ['write-content']
        },
        {
          action: uploadAction,
          filter:
            {
              action: filterUploadCommand,
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-upload-folder',
          name: 'upload-folder',
          privileges: ['bind']
        },
        {
          action: recent,
          filter:
            {
              action: filterRecentCommand,
              neededMethods: ['REPORT'],
              resourceTypes: ['collection']
            },
          label: 'label-recent',
          name: 'recent',
          privileges: ['read']
        },
        {
          action: showGroups,
          filter:
            {
              action: filterShowGroupsCommand,
              neededMethods: ['PROPFIND'],
              resourceTypes: ['principal']
            },
          label: 'label-show-groups',
          name: 'show-groups',
          privileges: ['read']
        },
        {
          filter: {action: filterSharingCommand, neededMethods: ['ACL']},
          label: 'label-sharing',
          name: 'sharing',
          privileges: ['read-acl', 'write-acl'],
          commands:
            [
              {
                action: sharingReadCommand,
                label: 'label-sharing-read',
                name: 'sharing-read'
              },
              {
                action: sharingWriteCommand,
                label: 'label-sharing-write',
                name: 'sharing-write'
              },
              {
                action: sharingReadVersionHistoryCommand,
                filter:
                  {resourceTypes: ['document', 'collection', '!principal']},
                label: 'label-sharing-read-version-history',
                name: 'sharing-read-version-history'
              },
              {
                action: sharingBindCommand,
                filter: {action: filterSharingCollectionCommand},
                label: 'label-sharing-bind',
                name: 'sharing-bind'
              },
              {
                action: sharingUnbindCommand,
                filter: {action: filterSharingCollectionCommand},
                label: 'label-sharing-unbind',
                name: 'sharing-unbind'
              },
              {
                action: sharingWriteDocumentsCommand,
                filter: {action: filterSharingCollectionCommand},
                label: 'label-sharing-write-documents',
                name: 'sharing-write-documents'
              }
            ]
        },
        {
          action: setMetaDataFixed,
          filter:
            {
              action: filterMetaDataCommand,
              neededMethods: ['PROPPATCH'],
              resourceTypes: ['document', 'principal', '!collection']
            },
          label: 'label-meta-data',
          name: 'meta-data',
          privileges: ['read', 'write']
        },
        {
          action: link,
          filter:
            {
              action: filterLinkCommand,
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-link',
          name: 'link',
          privileges: ['bind']
        },
        {
          action: checkInAction,
          filter: {action: filterCheckInCommand},
          label: 'label-check-in',
          name: 'check-in',
          privileges: ['checkin', 'read']
        },
        {
          action: checkOut,
          filter: {action: filterCheckOutCommand, neededMethods: ['CHECKOUT']},
          label: 'label-check-out',
          name: 'check-out',
          privileges: ['checkout', 'read']
        },
        {
          action: uncheckOut,
          filter: {action: filterCheckInCommand, neededMethods: ['UNCHECKOUT']},
          label: 'label-uncheck-out',
          name: 'uncheck-out',
          privileges: ['uncheckout', 'read']
        },
        {
          action: showVersions,
          filter:
            {
              action: filterVersionsCommand,
              neededMethods: ['REPORT'],
              resourceTypes: ['collection', 'document', '!principal']
            },
          label: 'label-versions',
          name: 'versions',
          privileges: ['read-version-history']
        },
        {
          action: annexes,
          filter:
            {
              action: filterRelationsCommand,
              neededMethods: ['PROPPATCH'],
              privileges: ['write-properties'],
              resourceTypes: ['document']
            },
          label: 'label-annexes',
          name: 'annexes',
          privileges: ['read', 'write']
        },
        {
          action: metaDataAddValueAction,
          filter: {action: filterMetaDataAddValueCommand},
          label: 'label-meta-data-add-value',
          name: 'meta-data-add-value',
          privileges: ['bind', 'read']
        },
        {
          action: sendMail,
          filter: {action: filterSendMailCommand},
          label: 'label-send-mail',
          name: 'send-mail',
          privileges: ['read']
        },
        {
          action: sendPass,
          filter:
            {
              action: filterSendPassCommand,
              resourceTypes: ['document', '!principal']
            },
          label: 'label-send-pass',
          name: 'send-pass',
          privileges: ['read']
        },
        {
          action: subscribeRSS,
          filter:
            {
              action: filterSubscribeRSSCommand,
              resourceTypes: ['collection', '!principal']
            },
          label: 'label-subscribe-rss',
          name: 'subscribe-rss',
          privileges: ['read']
        },
        {
          action: showLink,
          label: 'label-show-link',
          name: 'show-link'
        },
        {
          action: showBindings,
          filter:
            {
              action: filterShowBindingsCommand,
              resourceTypes: ['collection', 'document', '!principal']
            },
          label: 'label-show-bindings',
          name: 'show-bindings'
        },
        {
          action: unlock,
          filter:
            {
              action: filterUnlockCommand,
              resourceTypes: ['document']
            },
          label: 'label-unlock',
          name: 'unlock'
        },
        {
          filter: {action: filterWorkflow},
          label: 'label-workflow',
          name: 'workflow',
          commands:
            [
              {
                action: newWorkflow,
                filter:
                  {
                    action: filterNewWorkflowCommand,
                    privileges: ['bind'],
                    resourceTypes: ['collection', '!principal']
                  },
                label: 'label-new-workflow',
                name: 'new-workflow'
              },
              {
                action: editWorkflow,
                filter:
                  {
                    action: filterEditWorkflowCommand,
                    neededMethods: ['PUT'],
                    privileges: ['read', 'write'],
                    resourceTypes: ['document', '!principal']
                  },
                label: 'label-edit-workflow',
                name: 'edit-workflow'
              },
              {
                action: attachWorkflow,
                filter:
                  {
                    action: filterAttachWorkflowCommand,
                    neededMethods: ['PROPPATCH'],
                    privileges: ['write-properties'],
                    resourceTypes: ['collection', 'document', '!principal']
                  },
                label: 'label-attach-workflow',
                name: 'attach-workflow'
              },
              {
                action: nextWorkflow,
                filter:
                  {
                    action: filterStartWorkflowCommand,
                    neededMethods: ['PROPPATCH'],
                    privileges: ['write-properties'],
                    resourceTypes: ['document', '!principal']
                  },
                label: 'label-start-workflow',
                name: 'start-workflow'
              },
              {
                action: nextWorkflow,
                filter:
                  {
                    action: filterNextWorkflowCommand,
                    neededMethods: ['PROPPATCH'],
                    privileges: ['write-properties'],
                    resourceTypes: ['document', '!principal']
                  },
                label: 'label-next-workflow',
                name: 'next-workflow'
              },
              {
                action: previousWorkflow,
                filter:
                  {
                    action: filterPreviousWorkflowCommand,
                    neededMethods: ['PROPPATCH'],
                    privileges: ['write-properties'],
                    resourceTypes: ['document', '!principal']
                  },
                label: 'label-previous-workflow',
                name: 'previous-workflow'
              },
              {
                action: chooseWorkflow,
                filter:
                  {
                    action: filterChooseWorkflowCommand,
                    neededMethods: ['PROPPATCH'],
                    privileges: ['write-properties'],
                    resourceTypes: ['document', '!principal']
                  },
                label: 'label-choose-workflow',
                name: 'choose-workflow'
              },
              {
                action: showWorkflowOverview,
                filter:
                  {
                    action: filterWorkflowOverviewCommand,
                    neededMethods: ['REPORT'],
                    resourceTypes: ['document', '!principal']
                  },
                label: 'label-workflow-overview',
                name: 'workflow-overview',
                privileges: ['read-version-history']
              }
            ]
        },
        {
          action: downloadZip,
          filter: {resourceTypes: ['collection', '!principal']},
          label: 'label-download-zip',
          name: 'download-zip',
          privileges: ['read']
        },
        {
          action: expandZip,
          filter: {resourceTypes: ['collection', '!principal']},
          label: 'label-expand-zip',
          name: 'expand-zip',
          privileges: ['bind']
        },
        {
          filter:
            {
              action: filterActivityCommand,
              resourceTypes: ['collection', 'document', 'principal']
            },
          label: 'label-activity',
          name: 'activity',
          commands:
            [
              {
                action: allActions,
                filter: {action: filterEventsCommand},
                label: 'label-all-actions',
                name: 'all-actions'
              },
              {
                action: actionsPerDay,
                filter: {action: filterStatisticsCommand},
                label: 'label-actions-per-day',
                name: 'actions-per-day'
              },
              {
                action: downloadsPerDay,
                filter: {action: filterStatisticsCommand},
                label: 'label-downloads-per-day',
                name: 'downloads-per-day'
              },
              {
                action: uploadsPerDay,
                filter: {action: filterStatisticsCommand},
                label: 'label-uploads-per-day',
                name: 'uploads-per-day'
              },
              {
                action: downloadsPerFolder,
                filter:
                  {
                    action: filterStatisticsCommand,
                    resourceTypes: ['collection']
                  },
                label: 'label-downloads-per-folder',
                name: 'downloads-per-folder'
              },
              {
                action: downloadsPerMinute,
                filter: {action: filterStatisticsCommand},
                label: 'label-downloads-per-minute',
                name: 'downloads-per-minute'
              },
              {
                action: averageResponseTimePerDay,
                filter: {action: filterStatisticsCommand},
                label: 'label-average-response-time-per-day',
                name: 'average-response-time-per-day'
              },
              {
                action: uniqueVisitors,
                filter:
                  {
                    action: filterStatisticsCommand,
                    resourceTypes: ['collection']
                  },
                label: 'label-unique-visitors',
                name: 'unique-visitors'
              }
            ]
        }
      ];

    var resources =
      {
        en:
          {
            'all': 'Everybody',
            'authenticated': 'Authenticated',
            'err-different-passwords': 'The passwords are different.',
            'err-email': 'Invalid e-mail address.',
            'err-exists': 'Already exists.',
            'err-expired':
              'Your session has expired. The page will be reloaded.',
            'err-locked': 'Locked by ',
            'err-missing-fields': 'Some fields are missing.',
            'err-missing-meta-data': 'Some meta-data is missing.',
            'err-zip': 'Please select a ZIP-file.',
            'label-actions-per-day': 'Actions per day',
            'label-activity': 'Activity',
            'label-all-actions': 'All actions',
            'label-annexes': 'Annexes',
            'label-attach-workflow': 'Attach workflow',
            'label-average-response-time-per-day': 'Average response time',
            'label-check-in': 'Check in',
            'label-check-out': 'Check out',
            'label-choose-workflow': 'Choose step',
            'label-comment': 'Comment',
            'label-copy': 'Copy',
            'label-cut': 'Cut',
            'label-delete': 'Delete',
            'label-delete-all': 'Delete all',
            'label-download': 'Download',
            'label-downloads-per-day': 'Downloads per day',
            'label-downloads-per-folder': 'Downloads per folder',
            'label-downloads-per-minute': 'Downloads per minute',
            'label-download-zip': 'Download as ZIP',
            'label-edit-document': 'Edit',
            'label-edit-user': 'Edit',
            'label-edit-workflow': 'Edit',
            'label-expand': 'Expand',
            'label-expand-zip': 'Expand ZIP',
            'label-link': 'Link',
            'label-meta-data': 'Meta-data',
            'label-meta-data-add-value': 'Add value',
            'label-mkcol': 'New folder',
            'label-new-document': 'New document',
            'label-new-group': 'New group',
            'label-new-user': 'New user',
            'label-new-workflow': 'Create new',
            'label-next-workflow': 'Next',
            'label-paste': 'Paste',
            'label-previous-workflow': 'Previous',
            'label-recent': 'Recent',
            'label-rename': 'Rename',
            'label-send-mail': 'Send e-mail',
            'label-send-pass': 'Send pass',
            'label-sharing': 'Sharing',
            'label-sharing-bind': 'Only add',
            'label-sharing-read': 'Read',
            'label-sharing-read-version-history': 'Read version history',
            'label-sharing-unbind': 'Only delete',
            'label-sharing-write': 'Write',
            'label-sharing-write-documents': 'Update documents',
            'label-show-bindings': 'Show all paths',
            'label-show-groups': 'Groups',
            'label-show-link': 'Show URL',
            'label-start-workflow': 'Start',
            'label-subscribe-rss': 'Subscribe to RSS',
            'label-trash': 'Dustbin',
            'label-uncheck-out': 'Revert check-out',
            'label-unique-visitors': 'Unique visitors',
            'label-unlock': 'Unlock',
            'label-uploads-per-day': 'Uploads per day',
            'label-upload-document': 'Update document',
            'label-upload-folder': 'Upload',
            'label-versions': 'Versions',
            'label-workflow': 'Workflow',
            'label-workflow-overview': 'Overview',
            'unauthenticated': 'Unauthenticated'
          },
        fr:
          {
            'all': 'Tout le monde',
            'authenticated': 'Authentif\u00e9',
            'err-different-passwords':
              'Les mots de passe sont diff\u00e9rents.',
            'err-email': 'Adresse e-mail incorrecte.',
            'err-exists': 'Existe d\u00e9j\u00e0.',
            'err-expired':
              'Votre session a expir\u00e9e. La page sera recharg\u00e9e.',
            'err-locked': 'R\u00e9serv\u00e9 par ',
            'err-missing-fields': 'Certain champs manquent.',
            'err-missing-meta-data': 'Certain meta-data manque.',
            'err-zip': 'S\u00e9lectionnez un fichier ZIP s.v.p.',
            'label-actions-per-day': 'T\u00e9l\u00e9chargements par jour',
            'label-activity': 'Activit\u00e9',
            'label-all-actions': 'T\u00e9l\u00e9chargements',
            'label-annexes': 'Annexes',
            'label-attach-workflow': 'Attacher workflow',
            'label-average-response-time-per-day':
               'Temps de r\u00e9sponse moyen',
            'label-check-in': 'Check in',
            'label-check-out': 'Check out',
            'label-choose-workflow': 'Choisissez \u00e9tape',
            'label-comment': 'Commentaire',
            'label-copy': 'Copier',
            'label-cut': 'Couper',
            'label-delete': 'Supprimer',
            'label-delete-all': 'Supprimer tout',
            'label-download': 'T\u00e9l\u00e9charger',
            'label-downloads-per-day':
              'T\u00e9l\u00e9chargements par jour (bas)',
            'label-downloads-per-folder':
              'T\u00e9l\u00e9chargements par r\u00e9pertoire (bas)',
            'label-downloads-per-minute':
              'T\u00e9l\u00e9chargements par minute (bas)',
            'label-download-zip': 'T\u00e9l\u00e9charger comme ZIP',
            'label-edit-document': 'Modifier',
            'label-edit-user': 'Modifier',
            'label-edit-workflow': 'Modifier',
            'label-expand': 'Etendre',
            'label-expand-zip': 'Etendre ZIP',
            'label-link': 'Lien',
            'label-meta-data': 'Meta-data',
            'label-meta-data-add-value': 'Ajouter une valeur',
            'label-mkcol': 'Nouveau r\u00e9pertoire',
            'label-new-document': 'Nouveau document',
            'label-new-group': 'Nouveau groupe',
            'label-new-user': 'Nouvel utilisateur',
            'label-new-workflow': 'Cr\u00e9er nouveau',
            'label-next-workflow': 'Suivant',
            'label-paste': 'Coller',
            'label-previous-workflow': 'Pr\u00e9c\u00e9dent',
            'label-recent': 'R\u00e9cent',
            'label-rename': 'Changer le nom',
            'label-send-mail': 'Envoyer courrier \u00e9lectronique',
            'label-send-pass': 'Envoyer passeport',
            'label-sharing': 'Partager',
            'label-sharing-bind': 'Seulement ajouter',
            'label-sharing-read': 'Consulter',
            'label-sharing-read-version-history':
              'Consulter l\'histoire de versions',
            'label-sharing-unbind': 'Seulement supprimer',
            'label-sharing-write': 'Modifier',
            'label-sharing-write-documents': 'Modifier documents',
            'label-show-bindings': 'Montrer tous les chemins',
            'label-show-groups': 'Groupes',
            'label-show-link': 'Montrer URL',
            'label-start-workflow': 'D\u00e9marrer',
            'label-subscribe-rss': 'Abonnez-vous au RSS',
            'label-trash': 'Poubelle',
            'label-uncheck-out': 'D\u00e9fair check-out',
            'label-unique-visitors': 'Visiteurs uniques',
            'label-unlock': 'Relacher r\u00e9servation',
            'label-uploads-per-day':
              'T\u00e9l\u00e9chargements par jour (haut)',
            'label-upload-document': 'Modifier document',
            'label-upload-folder': 'T\u00e9l\u00e9charger',
            'label-versions': 'Versions',
            'label-workflow': 'Workflow',
            'label-workflow-overview': 'Vue d\'ensemble',
            'unauthenticated': 'Ne pas authentif\u00e9'
          },
        nl:
          {
            'all': 'Iedereen',
            'authenticated': 'Geauthenticeerd',
            'err-different-passwords': 'De wachtwoorden zijn verschillend.',
            'err-email': 'Ongeldig e-mailadres.',
            'err-exists': 'Bestaat al.',
            'err-expired':
              'Uw sessie is verlopen. De pagina zal worden herladen.',
            'err-locked': 'Gereserveerd door ',
            'err-missing-fields': 'Sommige velden ontbreken.',
            'err-missing-meta-data': 'Sommige meta-data ontbreekt.',
            'err-zip': 'Selecteer een ZIP-bestand a.u.b.',
            'label-actions-per-day': 'Acties per day',
            'label-activity': 'Activiteit',
            'label-all-actions': 'Alle acties',
            'label-annexes': 'Bijlagen',
            'label-attach-workflow': 'Workflow aanhechten',
            'label-average-response-time-per-day': 'Gemiddelde responsetijd',
            'label-check-in': 'Inchecken',
            'label-check-out': 'Uitchecken',
            'label-choose-workflow': 'Kies stap',
            'label-comment': 'Commentaar',
            'label-copy': 'Kopieer',
            'label-cut': 'Knip',
            'label-delete': 'Verwijder',
            'label-delete-all': 'Verwijder alles',
            'label-download': 'Download',
            'label-downloads-per-day': 'Downloads per dag',
            'label-downloads-per-folder': 'Downloads per map',
            'label-downloads-per-minute': 'Downloads per minuut',
            'label-download-zip': 'Downloaden als ZIP',
            'label-edit-document': 'Aanpassen',
            'label-edit-user': 'Aanpassen',
            'label-edit-workflow': 'Aanpassen',
            'label-expand': 'Expandeer',
            'label-expand-zip': 'Expandeer ZIP',
            'label-link': 'Link',
            'label-meta-data': 'Meta-data',
            'label-meta-data-add-value': 'Voeg waarde toe',
            'label-mkcol': 'Nieuwe map',
            'label-new-document': 'Nieuw document',
            'label-new-group': 'Nieuwe groep',
            'label-new-user': 'Nieuwe gebruiker',
            'label-new-workflow': 'Nieuwe maken',
            'label-next-workflow': 'Volgende',
            'label-paste': 'Plak',
            'label-previous-workflow': 'Vorige',
            'label-recent': 'Recent',
            'label-rename': 'Naam wijzigen',
            'label-send-mail': 'Verzend e-mail',
            'label-send-pass': 'Verzend pas',
            'label-sharing': 'Delen',
            'label-sharing-bind': 'Alleen toevoegen',
            'label-sharing-read': 'Raadplegen',
            'label-sharing-read-version-history':
              'Versiegeschiedenis raadplegen',
            'label-sharing-unbind': 'Alleen verwijderen',
            'label-sharing-write': 'Wijzigen',
            'label-sharing-write-documents': 'Documenten aanpassen',
            'label-show-bindings': 'Toon alle paden',
            'label-show-groups': 'Groepen',
            'label-show-link': 'URL tonen',
            'label-start-workflow': 'Start',
            'label-subscribe-rss': 'Abonneren op RSS',
            'label-trash': 'Vuilbak',
            'label-unique-visitors': 'Unieke bezoekers',
            'label-uncheck-out': 'Uitchecken ongedaan maken',
            'label-unlock': 'Reservatie vrijgeven',
            'label-uploads-per-day': 'Uploads per dag',
            'label-upload-document': 'Document aanpassen',
            'label-upload-folder': 'Opladen',
            'label-versions': 'Versies',
            'label-workflow': 'Workflow',
            'label-workflow-overview': 'Overzicht',
            'unauthenticated': 'Niet geauthenticeerd'
          }
      };

    var standardProperties =
      [
        {namespaceURI: $.pincette.webdav.dav, localName: 'auto-version'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'checked-out'},
        {
          namespaceURI: $.pincette.webdav.dav,
          localName: 'current-user-privilege-set'
        },
        {namespaceURI: $.pincette.webdav.dav, localName: 'displayname'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'getcontentlength'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'getcontenttype'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'getlastmodified'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'resourcetype'},
        {namespaceURI: $.pincette.webdav.re, localName: 'meta'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'preview-set'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'lockdiscovery'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'owner'},
        {namespaceURI: $.pincette.webdav.dav, localName: 'supported-method-set'}
      ];



    State = function(){}



    State.prototype.abortSearch = function()
    {
      if (this.runningSearch)
      {
        this.runningSearch.abort();
        this.runningSearch = null;
      }
    }



    State.prototype.addActivity = function()
    {
      ++this.activities;
      this.loading.show();
    }



    State.prototype.clearActivity = function()
    {
      this.activities = 0;
      this.loading.hide();
    }



    State.prototype.launchAction = function(action, response)
    {
      var command = findCommand(commands, action);

      if (command && command.action)
      {
        command.action(response, this);
      }
    }



    State.prototype.loadFolder = function(url)
    {
      loadFolder(url, false, this);
    }



    State.prototype.refresh = function(url, success, failure)
    {
      this.dav.
        getProperties(url, getStandardProperties(this), success, failure);
    }



    State.prototype.removeActivity = function()
    {
      if (this.activities > 0)
      {
        --this.activities;
        this.loading.hide();
      }
    }



    State.prototype.search = function(expressions, sortFields)
    {
      executeSearch(this.options.searchUrl, expressions, sortFields, this);
    }



    function actionsPerDay(response, state)
    {
      getStatistics
      (
        response,
        {
          count: true,
          operations: ['get', 'update'],
          groupBy: ['day'],
          orderBy: ['day']
        },
        {
          x: {field: 'day', type: 'date'},
          y: {field: 'count', type: 'integer'}
        },
        state
      );
    }



    function addCancelEvents(state)
    {
      $('*', state.pincette).
        filter(function(){return !$(this).data('pincette-registered-keyup');}).
        data('pincette-registered-keyup', true).
        keyup
        (
          function(e)
          {
            if (e.keyCode == '27')
            {
              state.abortSearch();
            }
          }
        );
    }



    function addComment(response, state)
    {
      var comment = state.comment.clone();

      $.pincette.util.ui.modal(comment, null, comment);
    }



    function addEnterEvents(state, form)
    {
      $('input', form).
        filter
        (
          function(index)
          {
            var field = state.options.fields[$(this).attr('name')];
            var result =
              !$(this).data('pincette-registered-keypress') &&
                (!field || !field.load);

            return result;
          }
        ).
        data('pincette-registered-keypress', true).
        keypress
        (
          function(e)
          {
            if (e.which == '13')
            {
              state.searchAction.click();
              $(this).blur();
            }
          }
        );
    }



    function addExpandedFolderAction(url, item, close, state)
    {
      item.click
      (
        function()
        {
          close();
          loadFolder(url, false, state);
        }
      );
    }



    function addFieldRow(element, state, action)
    {
      var row = $.pincette.util.getClosestRow(element);
      var newRow = row.clone();

      row.after(newRow);

      if (action)
      {
        action(newRow, state);
      }

      return newRow;
    }



    function addFolderExpansionEntries(url, tree, level, template, close, state)
    {
      var entries = $.pincette.util.getKeys(tree);

      entries.sort();

      for (var i = 0; i < entries.length; ++i)
      {
        if (entries[i] != 'path' && entries[i] != 'blocked')
        {
          var item =
            template.clone().removeClass('template').
              appendTo(template.parent());

          $.pincette.webdav.ui.
            replaceData(item, {name: entries[i]}, {}, state.dav);

          for (var j = 0; j < level; ++j)
          {
            $('.name', item).prepend('<div class="pincette-indent"></div>');
          }

          item.show();

          addExpandedFolderAction
          (
            url + tree[entries[i]].path,
            item,
            close,
            state
          );

          addFolderExpansionEntries
          (
            url,
            tree[entries[i]],
            level + 1,
            template,
            close,
            state
          );
        }
      }
    }



    function addGroup(template, state, group)
    {
      var item = template.clone();

      item.removeClass('template');

      $.pincette.webdav.ui.replaceData
      (
        item,
        {name: group ? group : options.allGroupsLabel},
        {},
        state.dav
      );

      if (group)
      {
        item.data('group', group);
      }

      template.parent().append(item);
      item.show();
      item.click(function(){showGroup(item.data('group'), state);});
    }



    function addWorkflowDecisions(responses, steps)
    {
      for (var i = 0; i < responses.length; ++i)
      {
        var id =
          $.pincette.webdav.getMetaDataValue(responses[i], 're_workflowStepId');
        var position = id ? getStepPosition(steps, id) : -1;
        var previousId =
          i > 0 ?
            $.pincette.webdav.
              getMetaDataValue(responses[i - 1], 're_workflowStepId') : null;
        var previousPosition =
          previousId ? getStepPosition(steps, previousId) : -1;

        responses[i].workflow_decision =
          position > previousPosition ||
            steps[steps.length - 1].status ==
              $.pincette.webdav.getMetaDataValue(responses[i], 're_status') ?
            'OK' : (position != -1 ? 'NOK' : null);
      }
    }



    function adjustHistory(url, state)
    {
      if (state.history.length > 0)
      {
        if (url == state.history[state.historyPosition])
        {
          return;
        }

        state.history = state.history.slice(0, ++state.historyPosition);
      }

      state.history.push(url);
      enableBackAndForward(state);
    }



    function advancedSearch(state)
    {
      var result = assignToState(state, 'advancedSearch', this);

      setFormWidgets(result, state);
      setPincetteFields(result);

      $.pincette.util.ui.editTableRows
      (
        result,
        function(row)
        {
          setPincetteFields(row);
          initializeSearchRow(row, state);
        },
        retainLastRowWithName,
        function(row)
        {
          $('input,select', row).val('');
          setResultHeight(state);
        }
      );

      return result;
    }



    function advancedSearchAction(state)
    {
      var result = assignToState(state, 'advancedSearchAction', this);

      result.click
      (
        function()
        {
          var search = false;

          $.pincette.util.ui.dialogBox
          (
            null,
            {
              onCancel:
                function(element)
                {
                  element.hide();
                },
              onOK:
                function(element)
                {
                  search = true;
                  element.hide();

                  return true;
                },
              afterClose:
                function()
                {
                  if (search)
                  {
                    search = false;
                    clearContains(state);
                    // The element is back in its original place.
                    state.searchAction.click();
                  }
                }
            },
            state.advancedSearch
          );
        }
      );

      return result;
    }



    function allActions(response, state)
    {
      function load()
      {
        state.dav.audit
        (
          response.href,
          fields,
          function(events)
          {
            close =
              showEvents
              (
                events,
                state,
                fields.after,
                fields.before,
                function(from, to)
                {
                  close();
                  fields.after = from;
                  fields.before = to;
                  load();
                }
              );
          }
        );
      }

      var close;
      var fields =
        {
          after: new Date(new Date().getTime() - 3 * 24 * 60 * 60 * 1000),
          before: new Date(),
          operations: ['get', 'update'],
          orderBy: ['!occurrence']
        };

      setEventSubject(response, fields);
      load();
    }



    function almostDue(response)
    {
      var value = $.pincette.webdav.getMetaDataValue(response, 're_dueDate');
      var result =
        value &&
          (
            $.pincette.util.parseISO8601(value).getTime() -
              new Date().getTime() <
              72 * 60 * 60 * 1000
          );

      return result;
    }



    function annexes(response, state)
    {
      relationsAction(response, 'annex', null, null, state);
    }



    function applyAclChanges(currentAcl, changes)
    {
      var result = new Array();

      for (var i = 0; i < currentAcl.length; ++i)
      {
        var found = false;

        for (var j = 0; j < changes.deleted.length && !found; ++j)
        {
          if ($.pincette.webdav.sameAce(currentAcl[i], changes.deleted[j]))
          {
            found = true;
          }
        }

        if (!found)
        {
          result.push(currentAcl[i]);
        }
      }

      for (var i = 0; i < changes.added.length; ++i)
      {
        var found = false;

        for (var j = 0; j < result.length && !found; ++j)
        {
          if ($.pincette.webdav.sameAce(changes.added[i], result[j]))
          {
            found = true;
          }
        }

        if (!found)
        {
          result.push(changes.added[i]);
        }
      }

      return result;
    }



    function asCollection(url)
    {
      var result =
        url != null && url.charAt(url.length - 1) != '/' ?
          (url + '/') : url;

      return result;
    }



    function assignToState(state, name, elements)
    {
      state[name] = getMostSpecific(state, elements);

      return state[name];
    }



    function attachWorkflow(response, state)
    {
      relationsAction
      (
        response,
        'workflow',
        function(response)
        {
          var result =
            response.isCollection ||
              response.getcontenttype ==
                'application/vnd.pincette.workflow+xml';

          return result;
        },
        1,
        state
      );
    }



    function averageResponseTimePerDay(response, state)
    {
      getStatistics
      (
        response,
        {
          avgDuration: true,
          groupBy: ['day'],
          orderBy: ['day']
        },
        {
          x: {field: 'day', type: 'date'},
          y: {field: 'avgDuration', type: 'duration'}
        },
        state
      );
    }



    function backAction(state)
    {
      assignToState(state, 'backAction', this).click
      (
        function()
        {
          goBack(state);
          enableBackAndForward(state);
        }
      );
    }



    function boxMenu(state)
    {
      assignToState(state, 'boxMenu', this);
    }



    function browse(button, input, foldersOnly, state)
    {
      $.pincette.util.ui.button($(button));

      button.click
      (
        function()
        {
          var copy = state.options.urlPicker.clone();

          if (foldersOnly)
          {
            $('.pincette-documents', copy).hide();
          }

          copy.pincette
          (
            'pickUrl',
            {
              rootUrl: state.options.urlPickerRootUrl,
              documentsOnly: false,
              onChoose:
                function(response)
                {
                  $(input).val($.pincette.util.url.getPath(response.href));
                }
            }
          );
        }
      );
    }



    function button()
    {
      $(this).each(function(){$.pincette.util.ui.button($(this))});
    }



    function callInitMethod(method, state, done)
    {
      if (!done[method])
      {
        var depends = initMethods[method].depends;

        if (depends)
        {
          for (i = 0; i < depends.length; ++i)
          {
            callInitMethod(depends[i], state, done);
          }
        }

        $('.' + methodToClass(method), state.pincette).
          pincette(method, state.options, state);
        done[method] = true;
      }
    }



    function canShowProperty(property, state)
    {
      var term = $.pincette.util.dc.getElementOrTerm(property);

      for (var i = 0; i < state.options.limitMetaFolders.length; ++i)
      {
        if
        (
          term == state.options.limitMetaFolders[i]		||
          property == state.options.limitMetaFolders[i]
        )
        {
          return true;
        }
      }

      return false;
    }



    function cancelAction(state)
    {
      assignToState(state, 'cancelAction', this).
        click(function(){state.abortSearch();});
      addCancelEvents(state);
    }



    function cascadeOptions(options)
    {
      options.sort
      (
        function(o1, o2)
        {
          // Least specific first.

          var result =
            o1.rootUrl && !o2.rootUrl ?
              -1 :
              (
                !o1.rootUrl && o2.rootUrl ?
                  1 :
                  (
                    o1.rootUrl && o2.rootUrl ?
                      o1.rootUrl.localeCompare(o2.rootUrl) : 0
                  )
              );

          return result;
        }
      );

      for (var i = 0; i < options.length; ++i)
      {
        if (options[i].rootUrl)
        {
          for (var j = i + 1; j < options.length; ++j)
          {
            if
            (
              $.pincette.util.startsWith(options[j].rootUrl, options[i].rootUrl)
            )
            {
              options[j] = $.pincette.util.inherit(options[i], options[j]);
            }
          }
        }
      }

      return options;
    }



    function chart(state)
    {
      assignToState(state, 'chart', this);
    }



    function checkAudit(state)
    {
      if (state.options.rootUrl)
      {
        state.dav.getProperties
        (
          state.options.rootUrl,
          [
            {
              namespaceURI: $.pincette.webdav.dav,
              localName: 'current-user-privilege-set'
            }
          ],
          function(response)
          {
            state.audit =
              response.privileges &&
                $.pincette.webdav.hasPrivilege(response.privileges, 'audit');
          },
          function(){state.audit = false;}
        );
      }
    }



    function checkDate(input)
    {
      input.change
      (
        function()
        {
          if (isValueSet($(this)))
          {
            if ($.pincette.util.isDate($(this).val().trim()))
            {
              $(this).removeClass('pincette-bad-value');
            }
            else
            {
              $(this).addClass('pincette-bad-value');
            }
          }
        }
      );
    }



    function checkExists(status, text, method, headers, state)
    {
      if (status == 403 || status == 405)
      {
        $.pincette.util.ui.messageBox
        (
          getResource("err-exists"),
          null,
          null,
          state.messageBox.clone()
        );
      }
      else
      {
        state.dav.getDefaultFailure()(status, text, method, headers);
      }
    }



    function checkIn(state)
    {
      assignToState(state, 'checkIn', this);
    }



    function checkInAction(response, state)
    {
      var box = state.checkIn.clone();

      $.pincette.util.ui.dialogBox
      (
        null,
        {
          onOK:
            function(element, userClose, reEnableDialogBox)
            {
              var comment =
                isValueSet($('textarea', box)) ?
                  $('textarea', box).val() : null;

              if
              (
                response.isCollection					&&
                $('input[name="propagate"]:checked', box).length == 1
              )
              {
                var reporter = state.reporter.clone();
      
                $.pincette.util.ui.modalWithClose(reporter);
                userClose();
      
                state.dav.checkIn
                (
                  response.href,
                  comment,
                  true,
                  function(response)
                  {
                    $.pincette.util.ui.addMessage
                    (
                      reporter,
                      $.pincette.webdav.
                        getPath(response.href, state.options.contextPath),
                      false
                    );
                  },
                  function(status, text, headers, method, url)
                  {
                    $.pincette.util.ui.addMessage
                    (
                      reporter,
                      $.pincette.webdav.getPath(url, state.options.contextPath)
                        + ': ' + text,
                      true
                    );
      
                    return true;
                  }
                );
              }
              else
              {
                state.dav.checkIn
                (
                  response.href,
                  comment,
                  false,
                  function()
                  {
                    userClose();
                    refreshSingle(response, state);
                  },
                  function(status, text, method, headers)
                  {
                    reEnableDialogBox();
                    state.dav.
                      getDefaultFailure()(status, text, method, headers);
                  }
                );
              }
            }
        },
        box
      );
    }



    function checkOut(response, state)
    {
      state.dav.checkOut
      (
        response.href,
        function(){refreshSingle(response, state);}
      );
    }



    function chooseWorkflow(response, state)
    {
      var url = $.pincette.webdav.getWorkflow(response);

      if (url)
      {
        state.dav.getWorkflow
        (
          url,
          function(steps)
          {
            selectWorkflowStep
            (
              response,
              steps,
              state,
              function(step)
              {
                stepWorkflow(response, null, step, state);
              }
            );
          }
        );
      }
    }



    function clearAction(state)
    {
      assignToState(state, 'clearAction', this).
        click(function(){clearSearchForms(state);});
    }



    function clearContains(state)
    {
      var contains = $('input[name="contains"]', state.simpleSearch);

      if (contains.val() == contains.attr('alt'))
      {
        contains.val('');
      }
    }



    function clearSearchForms(state)
    {
      $('input[type!="hidden"]', state.advancedSearch).each
      (
        function()
        {
          $(this).val(null);
        }
      );

      $('input', state.advancedSearch).removeClass('pincette-bad-value');

      $('input[type!="hidden"]', state.simpleSearch).each
      (
        function()
        {
          $(this).val(null);
        }
      );

      $('input', state.simpleSearch).removeClass('pincette-bad-value');
      state.searchAction.attr('disabled', false);
    }



    function closeForm(state)
    {
      if (!state.showFormAction.hasClass('closed'))
      {
        state.forms.hide();
        setResultHeight(state);
        state.showFormAction.addClass('closed');
      }
    }



    function comment(state)
    {
      assignToState(state, 'comment', this);
    }



    function compareMetaData(r1, r2, field, state)
    {
      var result =
        (!r1.meta || !r1.meta[field]) && r2.meta && r2.meta[field] ?
          1 :
          (
            r1.meta && r1.meta[field] && (!r2.meta || !r2.meta[field]) ?
              -1 :
              (
                r1.meta && r1.meta[field] && r2.meta && r2.meta[field] ?
                  (
                    state.options.fields[field] &&
                      (
                        state.options.fields[field].type == 'date' ||
                          state.options.fields[field].type == 'time'
                      ) ?
                      (
                        $.pincette.util.parseISO8601(r1.meta[field][0]).
                          getTime() -
                          $.pincette.util.parseISO8601(r2.meta[field][0]).
                            getTime()
                      ) :
                      (
                        state.options.fields[field] &&
                          state.options.fields[field].type == 'number' ?
                          (
                            Number(r1.meta[field][0]) -
                              Number(r2.meta[field][0])
                          ) :
                          r1.meta[field].join(',').toLocaleLowerCase().
                            localeCompare
                            (
                              r2.meta[field].join(',').toLocaleLowerCase()
                            )
                      )
                  ) : 0
              )
          );

      return result;
    }



    function comparePasswords(element, state)
    {
      if
      (
        $('input[name="password"]', element).val().trim() !=
          $('input[name="again"]', element).val().trim()
      )
      {
        $.pincette.util.ui.messageBox
        (
          getResource("err-different-passwords"),
          null,
          null,
          state.messageBox.clone()
        );

        return false;
      }

      return true;
    }



    function confirmationBox(state)
    {
      assignToState(state, 'confirmationBox', this);
    }



    function copy(response, state)
    {
      state.cutClipboard = null;
      state.copyClipboard = response;
    }



    function count(state)
    {
      assignToState(state, 'count', this);
    }



    function createMenu(element, createItems, state)
    {
      var menu = $('.pincette-response-menu', element);
      var boxMenu = $('.pincette-response-box-menu', element);

      $.pincette.util.ui.menu(menu, createItems);

      $.pincette.util.ui.boxMenu
      (
        boxMenu,
        function(){return state.boxMenu.clone();},
        createItems
      );
    }



    function createMenuItem(command, response, parentFolder, state)
    {
      var item =
        selectCommand(response, state, command) &&
          isAuthorizedCommand(command, response, parentFolder) ?
          {label: getResource(command.label)} : null;

      if (item)
      {
        if (command.action)
        {
          item.action =
            function(element)
            {
              if (command.action)
              {
                command.action(response, state);
              }
            };
        }

        if (command.commands)
        {
          item.createItems =
            function()
            {
              var result =
                createMenuItems
                (
                  command.commands,
                  response,
                  parentFolder,
                  state
                );

              return result;
            };
        }
      }

      return item;
    }



    function createMenuItems(commands, response, parentFolder, state)
    {
      var items = new Array();

      for (var i = 0; i < commands.length; ++i)
      {
        var item = createMenuItem(commands[i], response, parentFolder, state);

        if (item)
        {
          items.push(item);
        }
      }

      return items;
    }



    function createResponseMenu(element, response, parentFolder, state)
    {
      createMenu
      (
        element,
        function()
        {
          var result =
            createMenuItems
            (
              state.options.commands,
              response,
              parentFolder,
              state
            );

          return result;
        },
        state
      );
    }



    function createUser(response, state)
    {
      var copy = state.newUser.clone();

      populateGroupField(copy, state);
      $.pincette.util.ui.
        setLanguageSelection($('select[name="language"]', copy));
      setPincetteFields(copy);

      browse
      (
        $('.pincette-browse', copy),
        $('input[name="home"]', copy),
        true,
        state
      );

      $.pincette.util.ui.dialogBox
      (
        null,
        {
          onOK:
            function(element, userClose, reEnableDialogBox)
            {
              var url =
                state.options.rootUrl + 'users/' +
                  encodeURIComponent
                  (
                    $('input[name="name"]', element).val().trim()
                  );

              state.dav.create
              (
                url,
                null,
                function()
                {
                  userClose();

                  updateUser
                  (
                    url,
                    element,
                    {},
                    state,
                    function(){refresh(response.href, state);},
                    reEnableDialogBox
                  );

                  var group = $('select[name="group"]', copy);

                  if (isValueSet(group))
                  {
                    state.dav.create
                    (
                      group.val() + $('input[name="name"]', copy).val().trim()
                    );
                  }
                },
                function(status, text, method, headers)
                {
                  checkExists(status, text, method, headers, state);
                  reEnableDialogBox();
                }
              );
            },
          validate: function(element){return validateNewUser(element, state);}
        },
        copy
      );
    }



    function cut(response, state)
    {
      state.cutClipboard = response;
      state.copyClipboard = null;
    }



    function dateBox(state)
    {
      assignToState(state, 'dateBox', this);
    }



    function deleteFieldRow(element, state, action)
    {
      var row = $.pincette.util.getClosestRow(element);

      if
      (
        $('input,select', row).attr('name') ==
          $('input,select', row.prev()).attr('name')
      )
      {
        row.remove();
      }
      else
      {
        $('input,select', row).val('');
      }

      if (action)
      {
        action(row, state);
      }
    }



    function deleteAll(response, state)
    {
      $.pincette.util.ui.confirm
      (
        function()
        {
          state.dav.listFolder
          (
            response.href,
            [{namespaceURI: $.pincette.webdav.dav, localName: 'displayname'}],
            function(responses)
            {
              state.dav.startChaining();

              for (var i = 0; i < responses.length; ++i)
              {
                state.dav.deleteUrl(responses[i].href);
              }

              refresh(response.href, state);
              state.dav.runChain();
            }
          );
        },
        null,
        state.confirmationBox.clone()
      );
    }



    function deleteUrl(response, state)
    {
      $.pincette.util.ui.confirm
      (
        function()
        {
          state.dav.deleteUrl
          (
            response.href,
            function()
            {
              refresh($.pincette.util.getParent(response.href), state);
            }
          );
        },
        null,
        state.confirmationBox.clone()
      );
    }



    function details(state)
    {
      assignToState(state, 'details', this);
    }



    function displayEventField(event, name, type)
    {
      var result =
        type == 'time' ?
          $.pincette.util.displayDateTime(event[name]) :
          (
            type == 'date' ?
              $.pincette.util.displayDate(event[name]) :
              (
                type == 'string' ?
                  $.pincette.util.addWrapPoints(event[name]) :
                  (type == 'duration' ? (event[name] + 'ms') : event[name])
              )
          );

      return result;
    }



    function displayField(name, value, state)
    {
      var result =
        state.options.fields[name] ?
          (
            state.options.fields[name].type == 'date' ?
              $.pincette.util.displayDate($.pincette.util.parseISO8601(value)) :
              (
                state.options.fields[name].type == 'time' ?
                  $.pincette.util.
                    displayDateTime($.pincette.util.parseISO8601(value)) :
                  value
              )
          ) : value;

      return result;
    }



    function displayGroups(responses, state)
    {
      var currentGroup;
      var template = $('.template', state);

      template.parent().children(':not(.template)').remove();
      template.hide();

      for (var i = 0; i < responses.length; ++i)
      {
        var group = responses[i].meta[options.groupBy];

        if (group && (!currentGroup || group[0] != currentGroup))
        {
          addGroup(template, state, group[0]);
          currentGroup = group[0];
        }
      }

      addGroup(template, state);
    }



    function displayPath(url, state)
    {
      var path = $.pincette.util.url.getPath(url);
      var rootPath = $.pincette.util.url.getPath(state.options.rootUrl);
      var template = $('.template', state.path);

      template.parent().children(':not(.template)').remove();
      template.hide();

      if (path.indexOf(rootPath) == 0)
      {
        var segments =
          $.pincette.util.getPathSegments(path.substring(rootPath.length));

        for (var i = 0; i < segments.length; ++i)
        {
          segments[i] = decodeURIComponent(segments[i]);

          var segment =
            template.clone().removeClass('template').
              addClass('pincette-path-segment');

          $.pincette.webdav.ui.replaceData
          (
            segment,
            {
              name:
                segments[i] == '.Trash' ?
                  getResource('label-trash') : segments[i]
            },
            {},
            state.dav
          );

          template.parent().append(segment);
          segment.show();

          var additionalPath = '';

          for (var j = 0; j <= i; ++j)
          {
            additionalPath += encodeURIComponent(segments[j]) + '/';
          }

          segment.data('url', state.options.rootUrl + additionalPath);

          segment.click
          (
            function()
            {
              loadFolder($(this).data('url'), false, state);
            }
          );
        }
      }
    }



    function displayResponse
    (
      response,
      baseUrl,
      parentFolder,
      parentElement,
      template,
      openItem,
      isSearchResult,
      state
    )
    {
      try
      {
        if
        (
          state.options.showHidden				||
          !$.pincette.util.startsWith
          (
            $.pincette.util.getLastPathSegment(response.href),
            '.'
          )
        )
        {
          response.displayElement = {};
          response.displayElement.baseUrl = baseUrl;
          response.displayElement.item =
            template ? template.clone().removeClass('template') : null;
          response.displayElement.openItem = openItem;
          response.displayElement.parentElement = parentElement;
          response.displayElement.parentFolder = parentFolder;
          response.displayElement.template = template;

          if (isSearchResult)
          {
            response.displayElement.item.
              addClass('pincette-is-search-result');
          }

          state.options.displayResultEntry
          (
            response,
            parentElement,
            state,
            response.displayElement.item
          );

          if (response.displayElement.item)
          {
            parentElement.append(response.displayElement.item);
          }
        }
      }

      catch (e)
      {
      }
    }



    function displayResultEntry(response, parent, state, item)
    {
      if (item)
      {
        $('.pincette-button', item).
          each(function(){$.pincette.util.ui.button($(this))});
        $.pincette.webdav.ui.
          replaceData(item, response, state.options.fields, state.dav);
        item.data('pincette-response', response);

        if (response.displayElement.openItem)
        {
          item.click
          (
            function(){response.displayElement.openItem(response, state);}
          );
        }

        item.addClass(resourceClass(response));

        if (pastDue(response))
        {
          item.addClass('pincette-past-due');
        }
        else
        {
          if (almostDue(response))
          {
            item.addClass('pincette-almost-due');
          }
        }

        handlePreview
        (
          $('.pincette-preview', item),
          response,
          response.displayElement.baseUrl,
          state
        );

        $('.pincette-show-details-action', item).pincette
        (
          'showDetailsAction',
          state.options,
          state,
          response
        );

        createResponseMenu
        (
          item,
          response,
          response.displayElement.parentFolder,
          state
        );

        if (response.checkedOut)
        {
          $('.pincette-checked-out', item).show();
        }
        else
        {
          $('.pincette-checked-out', item).hide();
        }

        if (response.locks && response.locks.length > 0)
        {
          $('.pincette-locked', item).show();
        }
        else
        {
          $('.pincette-locked', item).hide();
        }

        if (hasMissingMetaData(response, state))
        {
          $('.pincette-missing-meta-data', item).
            show().
            click
            (
              function(e)
              {
                e.stopPropagation();

                $.pincette.util.ui.messageBox
                (
                  getResource("err-missing-meta-data"),
                  null,
                  null,
                  state.messageBox.clone()
                );
              }
            );
        }
        else
        {
          $('.pincette-missing-meta-data', item).hide();
        }

        if (state.options.afterDisplayResultEntry)
        {
          state.options.afterDisplayResultEntry(response, parent, state, item);
        }

        item.show();
      }
    }



    function displayResultSet
    (
      responses,
      element,
      baseUrl,
      openItem,
      state,
      parentFolder,
      isSearchResult
    )
    {
      var prepared = prepareDisplayResultSet(element);

      for (var i = 0; i < responses.length; ++i)
      {
        displayResponse
        (
          responses[i],
          baseUrl,
          parentFolder,
          prepared.parent,
          prepared.template,
          openItem,
          isSearchResult,
          state
        );
      }

      $.pincette.util.ui.setListIndex(element);
      prepared.parent.append(prepared.template);
    }



    function documentItems(state)
    {
      var result =
        $('.template', state.documents).siblings().
          filter(function(){return !$(this).hasClass('pincette-list-index');});

      return result;
    }



    function documents(state)
    {
      assignToState(state, 'documents', this);
    }



    function download(response, state)
    {
      $.pincette.util.openLocally(response.href + '?download');
    }



    function downloadsPerDay(response, state)
    {
      getStatistics
      (
        response,
        {
          count: true,
          operations: ['get'],
          groupBy: ['day'],
          orderBy: ['day']
        },
        {
          x: {field: 'day', type: 'date'},
          y: {field: 'count', type: 'integer'}
        },
        state
      );
    }



    function downloadsPerFolder(response, state)
    {
      getStatistics
      (
        response,
        {
          count: true,
          operations: ['get'],
          groupBy: ['path'],
          orderBy: ['path']
        },
        {
          x: {field: 'path', type: 'string'},
          y: {field: 'count', type: 'integer'}
        },
        state,
        function(events)
        {
          var accumulated = {};
          var parent = $.pincette.util.url.getPath(response.href);
          var parentSegments = $.pincette.util.getPathSegments(parent);
          var result = new Array();

          for (var i = 0; i < events.length; ++i)
          {
            var segments = $.pincette.util.getPathSegments(events[i].path);

            if (segments.length > parentSegments.length + 1) // In subfolder.
            {
              var path = parent + segments[parentSegments.length] + '/';

              if (accumulated[path])
              {
                accumulated[path] += events[i].count;
              }
              else
              {
                accumulated[path] = events[i].count;
              }
            }
          }

          for (var p in accumulated)
          {
            result.push({path: p, count: accumulated[p]});
          }

          result.sort(function(e1, e2){return e1.path.localeCompare(e2.path);});

          return result;
        }
      );
    }



    function downloadsPerMinute(response, state)
    {
      getStatistics
      (
        response,
        {
          after: new Date(new Date().getTime() - 60 * 60 * 1000),
          count: true,
          operations: ['get'],
          groupBy: ['minute'],
          orderBy: ['minute']
        },
        {
          x: {field: 'minute', type: 'time'},
          y: {field: 'count', type: 'integer'}
        },
        state
      );
    }



    function downloadZip(response, state)
    {
      $.pincette.util.
        openLocally(response.href + '?type=application%2fzip&depth=infinity');
    }



    function editDocument(response, state)
    {
      if (state.options.editors[response.getcontenttype])
      {
        window.open
        (
          $.pincette.util.url.resolveUrl
          (
            location.href,
            state.options.editors[response.getcontenttype]
          ) + '?path=' +
            encodeURIComponent($.pincette.util.url.getPath(response.href))
        );
      }
    }



    function editUser(url, state, done)
    {
      if (!state.user || state.user.length == 0)
      {
        return;
      }

      var current= {};

      state.dav.propfind
      (
        url,
        [
          {namespaceURI: $.pincette.webdav.dav, localName: 'displayname'},
          {namespaceURI: $.pincette.webdav.re, localName: 'backup'},
          {namespaceURI: $.pincette.webdav.re, localName: 'email'},
          {namespaceURI: $.pincette.webdav.re, localName: 'home'},
          {namespaceURI: $.pincette.webdav.re, localName: 'language'}
        ],
        '0',
        false,
        function(response)
        {
          current.displayname = response.displayname;
          current.backup = response.backup;
          current.email = response.email;
          current.home = response.home;
          current.language = response.language;
        },
        null,
        function()
        {
          var copy = state.user.clone();

          $.pincette.util.ui.
            setLanguageSelection($('select[name="language"]', copy));

          populateUsersField
          (
            copy,
            'backup',
            state,
            function(select)
            {
              if (current.backup)
              {
                setUserFieldValue(select, current.backup);
              }
            }
          );

          browse
          (
            $('.pincette-browse', copy),
            $('input[name="home"]', copy),
            true,
            state
          );

          $('input[name="displayname"]', copy).val(current.displayname);

          if (current.email)
          {
            $('input[name="email"]', copy).val(current.email);
          }

          if (current.home)
          {
            $('input[name="home"]', copy).
              val($.pincette.util.url.getPath(current.home));
          }

          if (current.language)
          {
            $('select[name="language"]', copy).val(current.language);
          }

          $.pincette.util.ui.dialogBox
          (
            null,
            {
              onOK:
                function(element, userClose, reEnableDialogBox)
                {
                  updateUser
                  (
                    url,
                    element,
                    current,
                    state,
                    function(url, properties)
                    {
                      userClose();

                      if (done)
                      {
                        done(url, properties);
                      }
                    },
                    reEnableDialogBox
                  );
                },
              onShow: function(){setPincetteFields(copy);},
              validate:
                function(element){return validateEditUser(element, state);}
            },
            copy
          );
        }
      );
    }



    function editUserCommand(response, state)
    {
      editUser
      (
        response.href,
        state,
        function(url, properties)
        {
          refreshSingle(response, state);
        }
      );
    }



    function editWorkflow(response, state)
    {
      state.dav.getUsers
      (
        state.options.rootUrl,
        state.options.contextPath,
        null,
        function(users)
        {
          $.pincette.util.sortCaseInsensitive(users);

          var displayNames = [];
          var displayNamesToNames = {};
          var namesToDisplayNames = {};
          var workflow = state.workflow.clone();
          var template = $('.template', workflow);

          for (var i = 0; i < users.length; ++i)
          {
            displayNames.push(users[i].displayname);
            displayNamesToNames[users[i].displayname] = users[i].name;
            namesToDisplayNames[users[i].name] = users[i].displayname;
          }

          state.dav.lock
          (
            response.href,
            -1,
            function(locks)
            {
              state.dav.get
              (
                response.href,
                function(status, responseText)
                {
                  if (responseText.length > 0)
                  {
                    var steps =
                      $.pincette.util.dom.selectElements
                      (
                        $.parseXML(responseText).documentElement,
                        'step'
                      );

                    for (var i = 0; i < steps.length; ++i)
                    {
                      var item = template.clone().removeClass('template');

                      initializeWorkflowStep
                      (
                        item,
                        steps[i],
                        displayNames,
                        namesToDisplayNames,
                        state
                      );

                      template.parent().append(item);
                    }

                    template.remove();
                  }
                  else
                  {
                    template.removeClass('template');

                    initializeWorkflowStep
                    (
                      template,
                      null,
                      displayNames,
                      null,
                      state
                    );
                  }

                  $.pincette.util.ui.editTableRows
                  (
                    workflow,
                    function(row)
                    {
                      initializeWorkflowStep
                      (
                        row,
                        null,
                        displayNames,
                        null,
                        state
                      );

                      setPincetteFields(row);
                      row.removeAttr('id');
                    }
                  );

                  $.pincette.util.ui.dialogBox
                  (
                    null,
                    {
                      onCancel:
                        function()
                        {
                          state.dav.unlock
                          (
                            response.href,
                            locks[0].token,
                            function(){refreshSingle(response, state);}
                          );
                        },
                      onOK:
                        function(element, userClose, reEnableDialogBox)
                        {
                          updateWorkflow
                          (
                            element,
                            response,
                            displayNamesToNames,
                            locks[0].token,
                            state,
                            userClose,
                            reEnableDialogBox
                          );
                        },
                      onShow: function(element){setPincetteFields(workflow);},
                      validate:
                        function(element){return validate(element, state);}
                    },
                    workflow
                  );
                }
              );
            }
          );
        }
      );
    }



    function enableBackAndForward(state)
    {
      if
      (
        state.history.length == 0				||
        state.historyPosition == state.history.length - 1
      )
      {
        state.forwardAction.addClass('disabled');
      }
      else
      {
        state.forwardAction.removeClass('disabled');
      }

      if (state.history.length == 0 || state.historyPosition == 0)
      {
        state.backAction.addClass('disabled');
      }
      else
      {
        state.backAction.removeClass('disabled');
      }
    }



    function events(state)
    {
      assignToState(state, 'events', this);
    }



    function executeSearch(url, expressions, sortFields, state)
    {
      var counter = 0;

      state.abortSearch();
      state.count.empty();
      state.folders.hide();
      state.documents.show();
      setResultHeight(state);

      var prepared = prepareDisplayResultSet(state.documents);

      state.runningSearch =
        state.dav.search
        (
          url,
          expressions,
          $.pincette.util.url.getPath(url) == '/groups/' ?
            [{namespaceURI: $.pincette.webdav.dav, localName: 'displayname'}] :
            getStandardProperties(state).concat
            (
              [{namespaceURI: $.pincette.webdav.re, localName: 'fragments'}]
            ),
          function(response)
          {
            if
            (
              !response.isCollection				&&
              (
                response.href.indexOf('/.Trash/') == -1		||
                $.pincette.util.endsWith(url, '/.Trash/')
              )							&&
              (
                !state.options.searchLimit			||
                ++counter <= state.options.searchLimit
              )
            )
            {
              displayResponse
              (
                response,
                state.options.searchUrl,
                null,
                prepared.parent,
                prepared.template,
                state.options.openDocument,
                true,
                state
              );
            }
          },
          null,
          function()
          {
            state.runningSearch = null;
            state.count.append(counter);
            $('input[name="contains"]', state.simpleSearch).focus();
            $.pincette.util.ui.setListIndex(state.documents);
            prepared.parent.append(prepared.template);

            if (state.searchDoneAction)
            {
              state.searchDoneAction.css('visibility', 'visible');
            }

            if (state.afterSearch)
            {
              state.afterSearch(url, state);
            }

            if (sortFields)
            {
              resortDocuments(sortFields, state);
            }
          }
        );
    }



    function expand(state)
    {
      assignToState(state, 'expand', this);
    }



    function expandAction(response, state)
    {
      function addEntry(response)
      {
        if (response.isCollection)
        {
          var blocked = false;
          var position = tree;
          var segments =
            $.pincette.util.getPathSegments
            (
              $.pincette.util.url.getPath(response.href).substring(path.length)
            );

          for (var i = 0; !blocked && i < segments.length; ++i)
          {
            var entry = position[segments[i]];

            if (!entry)
            {
              entry = {};
              position[segments[i]] = entry;
              entry.path = '';

              for (var j = 0; j <= i; ++j)
              {
                entry.path += encodeURIComponent(segments[j]) + '/';
              }

              entry.blocked =
                i == segments.length - 1 &&
                  (
                    !response.privileges ||
                      !$.pincette.webdav.
                        hasPrivilege(response.privileges, 'read') ||
                      (
                        !state.options.showHidden &&
                          $.pincette.util.startsWith(segments[i], '.')
                      )
                  );
            }
            else
            {
              if (entry.blocked)
              {
                blocked = true;
              }
            }

            position = entry;
          }
        }
      }

      function loaded()
      {
        showFolderExpansion(response.href, tree, state);
      }

      var path = $.pincette.util.url.getPath(response.href);
      var properties =
        [
          {
            namespaceURI: $.pincette.webdav.dav,
            localName: 'current-user-privilege-set'
          },
          {namespaceURI: $.pincette.webdav.dav, localName: 'displayname'},
          {namespaceURI: $.pincette.webdav.dav, localName: 'resourcetype'}
        ];
      var tree = {};

      if (path == '/groups/')
      {
        state.dav.propfind
        (
          response.href,
          properties,
          'infinity',
          false,
          addEntry,
          null,
          loaded
        );
      }
      else
      {
        state.dav.search
        (
          response.href,
          [
            {
              field: 'collection',
              op: '=',
              value: 'true'
            }
          ],
          properties,
          addEntry,
          null,
          loaded
        );
      }
    }



    function expandZip(response, state)
    {
      uploadFiles(response, true, state);
    }



    function fillPrincipalSelector(selector, all, selected, selectedClass)
    {
      all = $.pincette.util.removeAll(all, selected);
      $.pincette.util.sortCaseInsensitive(all);
      $.pincette.util.sortCaseInsensitive(selected);
      fillPrincipals(all, selector, 'all', selectedClass);
      fillPrincipals(selected, selector, selectedClass, 'all');
    }



    function fillPrincipals(principals, selector, className, otherClassName)
    {
      var principalsElement = $('.' + className + ' .principals', selector);
      var template = $('.' + className + ' .template', selector);

      for (var i = 0; i < principals.length; ++i)
      {
        var entry =
          template.clone().removeClass('template').appendTo(principalsElement);

        entry.add(entry.children()).filter('.name').
          append(principals[i].getDisplayname());
        entry.data('pincette-principal', principals[i]);

        entry.click
        (
          function()
          {
            movePrincipal($(this), selector, className, otherClassName);
          }
        );
      }
    }



    function filterActivityCommand(response, state, command)
    {
      var result =
        state.audit &&
          (
            (state.events && state.events.length > 0) ||
              (state.statistics && state.statistics.length > 0)
          ) &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isPropertyValue(response.href, state.options.contextPath) &&
          (
            !$.pincette.webdav.
              inGroups(response.href, state.options.contextPath) ||
            !response.isCollection
          );

      return result;
    }



    function filterAttachWorkflowCommand(response, state, command)
    {
      var result =
        state.relations && state.relations.length > 0 &&
          response.autoVersion && !response.meta.re_workflowStepId;

      return result;
    }



    function filterCheckInCommand(response, state, command)
    {
      var result =
        state.checkIn && state.checkIn.length > 0 &&
          (
            response.checkedOut ||
              (response.isCollection && command.name == 'check-in')
                // For propagation.
          );

      return result;
    }



    function filterCheckOutCommand(response, state, command)
    {
      var result =
        response.autoVersion && !response.checkedOut &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isPropertyValue(response.href, state.options.contextPath);

      return result;
    }



    function filterChooseWorkflowCommand(response, state, command)
    {
      var result =
        state.workflowStep && state.workflowStep.length > 0 &&
          state.workflowSteps && state.workflowSteps.length > 0 &&
          response.autoVersion && $.pincette.webdav.hasWorkflow(response);

      return result;
    }



    function filterCopyCutCommand(response, state, command)
    {
      var result =
        response.href != state.options.rootUrl &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath) &&
          !$.pincette.webdav.isActivity(response) &&
          !$.pincette.webdav.isView(response) &&
          (
            !$.pincette.webdav.isPrincipal(response) ||
              (
                response.name != 'system' &&
                  (
                    command.name == 'copy' ||
                      (
                        !$.pincette.webdav.
                          inUsers(response.href, state.options.contextPath) &&
                          !$.pincette.webdav.isGroups
                          (
                            $.pincette.util.getParent(response.href),
                            state.options.contextPath
                          )
                      )
                  )
              )
          );

      return result;
    }



    function filterDeleteAllCommand(response, state, command)
    {
      return $.pincette.webdav.isTrash(response.href);
    }



    function filterDeleteCommand(response, state, command)
    {
      var result =
        response.href != state.options.rootUrl &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isSystemUser(response.href, state.options.contextPath);

      return result;
    }



    function filterEditDocumentCommand(response, state, command)
    {
      var result =
        state.options.editors &&
          state.options.editors[response.getcontenttype] &&
          (!response.locks || response.locks.length == 0);

      return result;
    }



    function filterEditUserCommand(response, state, command)
    {
      return state.user && state.user.length > 0 && !response.isCollection;
    }



    function filterEditWorkflowCommand(response, state, command)
    {
      var result =
        state.workflow && state.workflow.length > 0 &&
          response.getcontenttype == 'application/vnd.pincette.workflow+xml';

      return result;
    }



    function filterEventsCommand(response, state, command)
    {
      return state.events && state.events.length > 0;
    }



    function filterExpandCommand(response, state, command)
    {
      var result =
        state.expand && state.expand.length > 0 &&
          (
            (
              !$.pincette.webdav.
                isSystemPath(response.href, state.options.contextPath) &&
                !$.pincette.webdav.
                  inMeta(response.href, state.options.contextPath)
            ) ||
              $.pincette.webdav.
                isGroups(response.href, state.options.contextPath)
          );

      return result;
    }



    function filterLinkCommand(response, state, command)
    {
      var result =
        !$.pincette.webdav.
          isSystemPath(response.href, state.options.contextPath) &&
          state.options.urlPicker && state.options.urlPicker.length > 0 &&
          !$.pincette.webdav.inMeta(response.href, state.options.contextPath);

      return result;
    }



    function filterMetaDataAddValueCommand(response, state, command)
    {
      var segments =
        $.pincette.util.getPathSegments
        (
          $.pincette.webdav.getPath(response.href, state.options.contextPath)
        );
      var result =
        state.metaDataAddValue && state.metaDataAddValue.length > 0 &&
          segments.length == 3 && segments[0] == 'meta' &&
          segments[1] == 'properties';

      return result;
    }



    function filterMetaDataCommand(response, state, command)
    {
      var result =
        (
          $.pincette.webdav.isPrincipal(response) &&
            state.userMetaData && state.userMetaData.length > 0
        ) || state.metaData && state.metaData.length > 0;

      return result;
    }



    function filterMkcolCommand(response, state, command)
    {
      var result =
        state.nameBox && state.nameBox.length > 0 &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath);

      return result;
    }



    function filterNewDocumentCommand(response, state, command)
    {
      var result =
        state.options.editors && state.newDocument &&
          state.newDocument.length > 0;

      return result;
    }



    function filterNewGroupCommand(response, state, command)
    {
      var result =
        state.nameBox && state.nameBox.length > 0 &&
          $.pincette.webdav.isGroups(response.href, state.options.contextPath);

      return result;
    }



    function filterNewUserCommand(response, state, command)
    {
      var result =
        state.newUser && state.newUser.length > 0 &&
          $.pincette.webdav.isUsers(response.href, state.options.contextPath);

      return result;
    }



    function filterNewWorkflowCommand(response, state, command)
    {
      var result =
        state.workflow && state.workflow.length > 0 &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.inMeta(response.href, state.options.contextPath);

      return result;
    }



    function filterNextWorkflowCommand(response, state, command)
    {
      var result =
        state.workflowStep && state.workflowStep.length > 0 &&
          response.autoVersion && response.meta.re_workflowStepId;

      return result;
    }



    function filterPasteDocumentCommand(response, state, command)
    {
      var source =
        state.copyClipboard ? state.copyClipboard : state.cutClipboard;
      var result =
        source &&
          $.pincette.util.
            intersect(source.resourceTypes, response.resourceTypes).length > 0
          && source.getcontenttype == response.getcontenttype;

      return result;
    }



    function filterPasteFolderCommand(response, state, command)
    {
      var path =
        $.pincette.webdav.getPath(response.href, state.options.contextPath);
      var source =
        state.copyClipboard ? state.copyClipboard : state.cutClipboard;
      var result =
        source && path != '/lost+found/' && path != '/meta/' &&
          path != '/meta/browse/' && path != '/meta/properties/' &&
          path != '/users/' &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath) &&
          (path != '/act/' || $.pincette.webdav.isActivity(source)) &&
          (path != '/view/' || $.pincette.webdav.isView(source)) &&
          (
            !$.pincette.webdav.isPrincipal(response) ||
              $.pincette.webdav.isPrincipal(source)
          ) &&
          (
            !$.pincette.webdav.
              isPropertyValue(response.href, state.options.contextPath) ||
              $.pincette.webdav.isDocument(source)
          );

      return result;
    }



    function filterPreviousWorkflowCommand(response, state, command)
    {
      var result =
        state.workflowStep && state.workflowStep.length > 0 &&
          response.autoVersion && response.meta.re_workflowStepId;

      return result;
    }



    function filterRecentCommand(response, state, command)
    {
      var path =
        $.pincette.webdav.getPath(response.href, state.options.contextPath);
      var result =
        path != '/act/' && path != '/lost+found/' && path != '/users/' &&
          path != '/view/' && path != '/meta/browse/' &&
          path != '/meta/properties/' &&
          !$.pincette.util.startsWith(path, '/groups/') &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath);

      return result;
    }



    function filterRelationsCommand(response, state, command)
    {
      return state.relations && state.relations.length > 0;
    }



    function filterRenameCommand(response, state, command)
    {
      var result =
        state.nameBox && state.nameBox.length > 0 &&
          response.href != state.options.rootUrl &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            inMetaBrowse(response.href, state.options.contextPath) &&
          (
            !$.pincette.webdav.isPrincipal(response) ||
              (
                response.name != 'system' &&
                (
                  !$.pincette.webdav.
                    inGroups(response.href, state.options.contextPath) ||
                    $.pincette.webdav.isGroups
                    (
                      $.pincette.util.getParent(response.href),
                      state.options.contextPath
                    )
                )
              )
          );

      return result;
    }



    function filterSendMailCommand(response, state, command)
    {
      return state.selectPrincipals && state.selectPrincipals.length > 0;
    }



    function filterSendPassCommand(response, state, command)
    {
      var result =
        state.options.passUrl && state.dateBox && state.dateBox.length > 0;

      return result;
    }



    function filterSharingCollectionCommand(response, state, command)
    {
      var result =
        $.pincette.webdav.isPureCollection(response, state.options.contextPath);

      return result;
    }



    function filterSharingCommand(response, state, command)
    {
      return state.sharing && state.sharing.length > 0;
    }



    function filterShowBindingsCommand(response, state, command)
    {
      var result =
        !$.pincette.webdav.
          isSystemPath(response.href, state.options.contextPath) &&
        !$.pincette.webdav.
          isProperty(response.href, state.options.contextPath) &&
        !$.pincette.webdav.
          isPropertyValue(response.href, state.options.contextPath);

      return result;
    }



    function filterShowGroupsCommand(response, state, command)
    {
      return state.expand && state.expand.length > 0 && !response.isCollection;
    }



    function filterStartWorkflowCommand(response, state, command)
    {
      var result =
        state.workflowStep && state.workflowStep.length > 0 &&
          response.autoVersion && !response.meta.re_workflowStepId &&
          $.pincette.webdav.hasWorkflow(response);

      return result;
    }



    function filterStatisticsCommand(response, state, command)
    {
      return state.statistics && state.statistics.length > 0;
    }



    function filterSubmenu(response, state, command)
    {
      if (command.commands)
      {
        for (var i = 0; i < command.commands.length; ++i)
        {
          if (selectCommand(response, state, command.commands[i]))
          {
            return true; // One enabled command is enough.
          }
        }
      }

      return false;
    }



    function filterSubscribeRSSCommand(response, state, command)
    {
      var result =
        !$.pincette.webdav.
          isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath);

      return result;
    }



    function filterUnlockCommand(response, state, command)
    {
      var result =
        response.locks && response.locks.length > 0 && response.locks[0].token;

      return result;
    }



    function filterUpdateDocumentCommand(response, state, command)
    {
      var result =
        state.options.uploadUrl && state.upload && state.upload.length > 0 &&
          $.pincette.webdav.canUpdate(response);

      return result;
    }



    function filterUploadCommand(response, state, command)
    {
      var result =
        state.options.uploadUrl && state.upload && state.upload.length > 0 &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isPropertyValue(response.href, state.options.contextPath) &&
          $.pincette.webdav.canUpdate(response);

      return result;
    }



    function filterVersionsCommand(response, state, command)
    {
      var result =
        response.autoVersion && state.versions && state.versions.length > 0 &&
          !$.pincette.webdav.
            isSystemPath(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isProperty(response.href, state.options.contextPath) &&
          !$.pincette.webdav.
            isPropertyValue(response.href, state.options.contextPath);

      return result;
    }



    function filterWorkflow(response, state, command)
    {
      var result =
        !$.pincette.webdav.inMeta(response.href, state.options.contextPath) &&
          filterSubmenu(response, state, command);

      return result;
    }



    function filterWorkflowOverviewCommand(response, state, command)
    {
      var result =
        state.workflowOverview && state.workflowOverview.length > 0 &&
          filterVersionsCommand(response, state, command);

      return result;
    }



    function findCommand(commands, name)
    {
      for (var i = 0; i < commands.length; ++i)
      {
        if (name == commands[i].name)
        {
          return commands[i];
        }

        if (commands[i].commands)
        {
          var result = findCommand(commands[i].commands, name);

          if (result)
          {
            return result;
          }
        }
      }

      return null;
    }



    function folders(state)
    {
      assignToState(state, 'folders', this);
    }



    function forms(state)
    {
      assignToState(state, 'forms', this);
    }



    function forwardAction(state)
    {
      assignToState(state, 'forwardAction', this).click
      (
        function()
        {
          goForward(state);
          enableBackAndForward(state);
        }
      );
    }



    function generateExpression(state)
    {
      var contains = null;
      var fullText = generateFullTextExpression(state);
      var result = new Array();

      if (fullText)
      {
        result.push(fullText);
      }

      if (state.options.extraSearchExpr)
      {
        result.push(state.options.extraSearchExpr);
      }

      $('*', state.advancedSearch).filter($.pincette.util.selectTableRows).each
      (
        function()
        {
          var expr = generateMetaDataExpression($(this), state);

          if (expr)
          {
            var booleanOperator =
              $(
                'select[name="boolean"]',
                $(this).prev('*').filter($.pincette.util.selectTableRows)
              );
            var operator =
              booleanOperator.val() ? booleanOperator.val() : 'and';

            if (operator == 'and')
            {
              result.push(expr);
            }
            else
            {
              if (result[result.length - 1].op != 'or')
              {
                var or = {};

                or.op = 'or';
                or.expressions = new Array();
                or.expressions.push(result[result.length - 1]);
                result[result.length - 1] = or;
              }

              result[result.length - 1].expressions.push(expr);
            }
          }
        }
      );

      return result;
    }



    function generateExpressionForInput(input, operator, state)
    {
      var expr = {};

      expr.originalField = input.attr('name');
      expr.value = getValue(expr.originalField, input.val().trim(), state);

      if ($.pincette.webdav.isTechnicalField(expr.originalField))
      {
        expr.field = expr.originalField;
      }
      else
      {
        var label =
          $.pincette.util.dc.
            getLabel(expr.originalField, $.pincette.util.getLocale());

        expr.field = 'meta.' + (label ? label : expr.originalField);
      }

      expr.op = operator;

      return expr;
    }



    function generateFieldTerm(term, field)
    {
      var result =
        term.charAt(0) == '+' || term.charAt(0) == '-' ?
          (term.charAt(0) + field + ':' + term.substring(1)) :
          (field + ':' + term);

      return result;
    }



    function generateFullTextExpression(state)
    {
      var value =
        $.pincette.util.ui.
          getFieldValue($('input[name="contains"]', state.simpleSearch));

      if (value == null || value.trim().length == 0)
      {
        return null;
      }

      value = value.trim();

      var expr = {};
      var path = $.pincette.util.url.getPath(state.options.searchUrl);

      if (path == '/groups/' || path == '/users/')
      {
        expr.field = 'displayname';
        expr.op = '=';
        expr.value = value;

        return expr;
      }

      var abstract = '';
      var contents = '';
      var name = '';
      var title = '';
      var values =
        getFullTextTerms
        (
          value.replace(/\u002a/g, '~').replace(/\++/g, '+').
            replace(/-+/g, '-').replace(/-\+/g, '+').replace(/\+-/g, '+').
            replace(/-\(*/g, '-').replace(/\+\(*/g, '+')
        );

      for (var i = 0; i < values.length; ++i)
      {
        if (isChecked('contents', state))
        {
          contents += ' ' + values[i];
        }

        if
        (
          values[i].toLowerCase() != 'and'	&&
          values[i].toLowerCase() != 'or'
        )
        {
          if (isChecked('title', state))
          {
            title += ' ' + generateFieldTerm(values[i], 'title');
          }

          if (isChecked('abstract', state))
          {
            abstract += ' ' + generateFieldTerm(values[i], 'abstract');
          }

          name += ' ' + generateFieldTerm(values[i], 'name');
        }
      }

      expr.value = '';

      if (contents.length > 0)
      {
        expr.value += contents;
      }

      if (title.length > 0)
      {
        expr.value += title;
      }

      if (abstract.length > 0)
      {
        expr.value += abstract;
      }

      expr.value += name;
      expr.field = 'contains';
      expr.op = '=';

      return expr;
    }



    function generateMetaDataExpression(tr, state)
    {
      var expr = null;
      var input = $('input[class!="not-meta-data"]', tr);

      if (input.length == 1)
      {
        if (isValueSet(input))
        {
          expr = generateExpressionForInput(input, '=', state);
        }
      }
      else
      {
        if (input.length == 2)
        {
          var from = input.first();
          var to = input.last();

          if (isValueSet(from) || isValueSet(to))
          {
            expr = {};
            expr.op = 'and';
            expr.expressions = new Array();

            if (isValueSet(from))
            {
              expr.expressions.
                push(generateExpressionForInput(from, '>=', state));
            }

            if (isValueSet(to))
            {
              expr.expressions.
                push(generateExpressionForInput(to, '<=', state));
            }
          }
        }
      }

      return expr;
    }



    function getCommand(name)
    {
      for (var i = 0; i < commands.length; ++i)
      {
        if (name == commands[i].name)
        {
          return commands[i];
        }
      }

      return null;
    }



    function getDatepickerRegional()
    {
      var locale = $.pincette.util.getLocale();
      var result =
        $.datepicker.regional[$.pincette.util.correctLanguageTag(locale)];

      if (!result && locale.indexOf('-') != -1)
      {
        result =
          $.datepicker.regional[locale.substring(0, locale.indexOf('-'))];
      }

      if (!result)
      {
        result = $.datepicker.regional[''];
      }

      return result;
    }



    function getFullTextTerms(s)
    {
      var startDouble = s.indexOf('"');
      var startSingle = s.indexOf('\'');
      var start =
        startDouble != -1 && startSingle != -1 ?
          Math.min(startDouble, startSingle) :
          (startDouble != -1 ? startDouble : startSingle);

      if (start != -1)
      {
        var end = s.indexOf(s.charAt(start), start + 1);

        if (end != -1)
        {
          if
          (
            start > 0				&&
            (
              s.charAt(start - 1) == '+'	||
              s.charAt(start - 1) == '-'
            )
          )
          {
            --start;
          }

          var result = removeEmpty(s.substring(0, start).split(' '));

          result.push(s.substring(start, end + 1));
          result = result.concat(getFullTextTerms(s.substring(end + 1)));

          return result;
        }

        result =
          getFullTextTerms
          (
            s.replace(s.charAt(start) == '"' ? /"/ : /'/, '')
          );

        return result;
      }

      return removeEmpty(s.split(' '));
    }



    function getGrantedPrincipals(aces, sharing, privilege, state, principals)
    {
      for (var i = 0; i < aces.length; ++i)
      {
        if
        (
          aces[i].grant 						&&
          aces[i].privilege.localName == privilege.localName		&&
          aces[i].privilege.namespaceURI == privilege.namespaceURI
        )
        {
          getPrincipal
          (
            aces[i].principal,
            state,
            function(principal){principals.push(principal);}
          );
        }
      }
    }



    function getLargestPreview(response)
    {
      if (!response.previews)
      {
        return null;
      }
    
      var preview = null;
    
      for (var i = 0; i < response.previews.length; ++i)
      {
        if
        (
          !preview							 ||
          Number(response.previews[i].width) > Number(preview.width)
        )
        {
          preview = response.previews[i];
        }
      }
    
      return preview;
    }



    function getMandatoryMetaData(state)
    {
      var result = new Array();

      if (state.metaData && state.metaData.length > 0)
      {
        $(
          'input.pincette-mandatory,textarea.pincette-mandatory,' +
            'select.pincette-mandatory',
          state.metaData
        ).each
          (
            function()
            {
              if (!$.pincette.util.inArray(result, $(this).attr('name')))
              {
                result.push($(this).attr('name'));
              }
            }
          );
      }

      return result;
    }



    function getMostSpecific(state, elements)
    {
      var result = $();
      var rootPath = $.pincette.util.url.getPath(state.options.rootUrl);

      elements.each
      (
        function()
        {
          var currentPath =
            selectMostSpecific(getPathsFromClass(result), rootPath);
          var path = selectMostSpecific(getPathsFromClass($(this)), rootPath);

          if
          (
            (
              path == null					||
              $.pincette.util.startsWith(rootPath, path)
            )							&&
            (
              result.length == 0				||
              (
                currentPath == null				&&
                path != null
              )							||
              (
                currentPath != null				&&
                path != null					&&
                path.length > currentPath.length
              )
            )
          )
          {
            // Force this in all circumstances.
            result.attr('style', 'display: none !important');
            result = $(this);
          }
          else
          {
            // Force this in all circumstances.
            $(this).attr('style', 'display: none !important');
          }
        }
      );

      return result;
    }



    function getOpenFolder(state)
    {
      return state.history[state.historyPosition];
    }



    function getOptionsFromPath(options)
    {
      var path = $.pincette.util.url.getParameters(location.href).path;

      if (!path || path.length == 0)
      {
        path = ['/'];
      }

      // From most to least specific.

      for (var i = options.length - 1; i >= 0; --i)
      {
        if
        (
          $.pincette.util.
            startsWith(path[0], $.pincette.util.url.getPath(options[i].rootUrl))
        )
        {
          return options[i];
        }
      }

      return {};
    }



    function getPathsFromClass(element)
    {
      var result = new Array();

      if (element.attr('class'))
      {
        var values = element.attr('class').split(' ');

        for (var i = 0; i < values.length; ++i)
        {
          if (values[i].length > 0 && values[i].charAt(0) == '/')
          {
            result.push(values[i]);
          }
        }
      }

      return result;
    }



    function getPrincipal(principal, state, done)
    {
      var result =
        new $.pincette.webdav.Principal
        (
          $.pincette.util.url.isUrl(principal) ||
            $.pincette.util.startsWith(principal, '/') ?
            decodeURIComponent($.pincette.util.getLastPathSegment(principal)) :
            (
              $.pincette.webdav.isReservedPrincipal(principal) ?
                getResource(principal) : principal
            ),
          principal
        );

      if (state && $.pincette.util.url.isUrl(principal))
      {
        state.dav.getProperties
        (
          principal,
          [{namespaceURI: $.pincette.webdav.dav, localName: 'displayname'}],
          function(response)
          {
            result.setDisplayname(response.displayname);
            result.response = response;

            if (done)
            {
              done(result);
            }
          },
          function(){},
          function(){return true;} // This was only an attempt.
        );
      }
      else
      {
        if (done)
        {
          done(result);
        }
      }

      return result;
    }



    function getPrincipalWithLabel(principals, label)
    {
      if (principals)
      {
        for (var i = 0; i < principals.length; ++i)
        {
          if (principals[i].displayname == label)
          {
            return principals[i];
          }
        }
      }

      return null;
    }



    function getResource(key)
    {
      var map =
        $.pincette.util.perLanguage(resources, $.pincette.util.getLocale());
      var value = map[key];
      var result = value ? value : key;

      return result;
    }



    function getSelectedPrincipals(selector, className)
    {
      var names = [];

      $('.' + className + ' .principals > *', selector).each
      (
        function()
        {
          if ($(this).data('pincette-principal'))
          {
            names.push($(this).data('pincette-principal'));
          }
        }
      );

      return names;
    }



    function getSortField(element)
    {
      var values = element.attr('class').split(' ');

      for (var i = 0; i < values.length; ++i)
      {
        if
        (
          values[i].charAt(0) == '['			&&
          values[i].charAt(values[i].length - 1) == ']'
        )
        {
          return values[i].substring(1, values[i].length - 1);
        }
      }

      return null;
    }



    function getStandardProperties(state)
    {
      var result =
        state.options.annexesInBackground ?
          standardProperties :
          standardProperties.concat
          (
            [{namespaceURI: $.pincette.webdav.re, localName: 'referring-to'}]
          );

      return result;
    }



    function getStatistics(response, fields, map, state, preprocess)
    {
      function load()
      {
        state.dav.audit
        (
          response.href,
          fields,
          function(events)
          {
            if (preprocess)
            {
              events = preprocess(events);
            }

            close =
              showStatistics
              (
                events,
                map,
                state,
                fields.after,
                fields.before,
                function(from, to)
                {
                  close();
                  fields.after = from;
                  fields.before = to;
                  load();
                }
              );
          }
        );
      }

      var close;

      if (!fields.after)
      {
        fields.after =
          new Date(new Date().getTime() - 30 * 24 * 60 * 60 * 1000);
      }

      if (!fields.before)
      {
        fields.before = new Date();
      }

      setEventSubject(response, fields);
      load();
    }



    function getStepPosition(steps, id)
    {
      for (var i = 0; i < steps.length; ++i)
      {
        if (steps[i].id == id)
        {
          return i;
        }
      }

      return -1;
    }



    function getValue(name, displayValue, state)
    {
      try
      {
        var result =
          state.options.fields[name] ?
            (
              state.options.fields[name].type == 'date' ?
                $.pincette.util.dateISO8601
                (
                  $.pincette.util.isDate(displayValue) ?
                    $.pincette.util.parseDate
                    (
                      displayValue, $.pincette.util.getShortDateFormat()
                    ) : $.pincette.util.parseISO8601(displayValue)
                ) :
                (
                  state.options.fields[name].type == 'time' ?
                    $.pincette.util.timestampISO8601
                    (
                      $.pincette.util.isTime(displayValue) ?
                        $.pincette.util.parseDate
                        (
                          displayValue,
                          $.pincette.util.getShortDateFormat(),
                          $.pincette.util.getTimeFormat()
                        ) : $.pincette.util.parseISO8601(displayValue)
                    ) : displayValue
                )
            ) : displayValue;

        return result;
      }

      catch (e)
      {
        return displayValue;
      }
    }



    function goBack(state)
    {
      if (state.historyPosition > 0)
      {
        loadFolder(state.history[--state.historyPosition], true, state);
      }
    }



    function goForward(state)
    {
      if (state.historyPosition < state.history.length - 1)
      {
        loadFolder(state.history[++state.historyPosition], true, state);
      }
    }



    function handlePreview(element, response, baseUrl, state)
    {
      if (!response.previews || !element || state.previewBox.length == 0)
      {
        return;
      }
    
      var preview = getLargestPreview(response);
    
      if (!preview)
      {
        return;
      }
 
      var img = $('img', element);

      img.click
      (
        function(e)
        {
          e.stopPropagation();
    
          var element = state.previewBox.clone();
          var img =
            $(
              '<img src="' +
                $.pincette.util.url.resolveUrl(baseUrl, preview.href) + '"/>'
            );

          element.append(img);
          img.height(Math.min($(window).height() - 40, preview.height));

          img.width
          (
            img.height() == preview.height ?
              preview.width :
              Math.round(preview.width * img.height() / preview.height)
          );

          element.height(img.outerHeight());
          element.width(img.outerWidth());

          var close =
            $.pincette.util.ui.modal
            (
              element,
              {
                onShow:
                  function(container, overlay, box)
                  {
                    container.click(function(){close();});
                    overlay.click(function(){close();});
                    overlay.show();
                    container.show();

                    box.show
                    (
                      function()
                      {
                        container.height(element.outerHeight());
                      }
                    );
                  }
              }
            );
        }
      );
    }



    function hasMissingMetaData(response, state)
    {
      if
      (
        response.resourceTypes						&&
        $.pincette.util.inArray(response.resourceTypes, 'document')	&&
        state.mandatoryMetaData
      )
      {
        for (var i = 0; i < state.mandatoryMetaData.length; ++i)
        {
          if (!response.meta || !response.meta[state.mandatoryMetaData[i]])
          {
            return true;
          }
        }
      }

      return false;
    }



    function hasNeededContentType(response, command)
    {
      var result =
        !command.filter.types ||
          (
            !response.isCollection &&
              $.pincette.util.
                inArray(command.filter.types, response.getcontenttype)
          );

      return result;
    }



    function hasNeededMethod(response, command)
    {
      var result =
        !command.filter.neededMethods ||
          (
            response.supportedMethods &&
              $.pincette.util.intersect
              (
                command.filter.neededMethods,
                response.supportedMethods
              ).length > 0
          );

      return result;
    }



    function hasNeededResourceType(response, command)
    {
      if (!response.resourceTypes)
      {
        return false;
      }

      if (!command.filter.resourceTypes)
      {
        return true;
      }

      var have = new Array();
      var haveNot = new Array();

      for (var i = 0; i < command.filter.resourceTypes.length; ++i)
      {
        if (command.filter.resourceTypes[i].charAt(0) == '!')
        {
          haveNot.push(command.filter.resourceTypes[i].substring(1));
        }
        else
        {
          have.push(command.filter.resourceTypes[i]);
        }
      }

      var result =
        $.pincette.util.intersect(have, response.resourceTypes).length > 0 &&
          $.pincette.util.
            intersect(haveNot, response.resourceTypes).length == 0;

      return result;
    }



    function helpAction(state)
    {
      assignToState(state, 'helpAction', this).
        click(function(){openHelp(state.options);});
    }



    function init(state)
    {
      state.pincette = this;

      var done = {};

      for (var method in initMethods)
      {
        if (initMethods[method].init)
        {
          callInitMethod(method, state, done);
        }
      }

      setResultHeight(state);
      preload(state);

      return this;
    }



    function initializeMetaDataRow(row, state)
    {
      setFormWidgets(row, state);
      setAutoComplete($('input', row), state);

      checkDate
      (
        $('input', row).
          filter(function(){return isDate($(this).attr('name'), state);})
      );
    }



    function initializeRelation(item, relation, filter, state, placeHolderText)
    {
      function setUrl(element, url)
      {
        element.empty();
        element.append($.pincette.util.addWrapPoints(url));

        if (url != placeHolderText)
        {
          element.removeClass('place-holder');
          element.data('pincette-url', url);
        }
      }

      var url = $('.url', item);

      url.addClass('place-holder');

      url.each
      (
        function()
        {
          var element = $(this);

          setUrl(element, relation ? relation.href : placeHolderText);

          element.click
          (
            function()
            {
              state.options.urlPicker.pincette
              (
                'pickUrl',
                {
                  rootUrl: state.options.urlPickerRootUrl,
                  sortDocuments: state.options.sortDocuments,
                  sortFolders: state.options.sortFolders,
                  documentsOnly: true,
                  filter: filter,
                  onChoose: function(response){setUrl(element, response.href);}
                }
              );
            }
          );
        }
      );
    }



    function initializeSearchRow(row, state)
    {
      setAutoComplete($('input', row), state);
      setFormWidgets(row, state);
      setResultHeight(state);
    }



    function initializeWorkflowStep
    (
      row,
      step,
      displayNames,
      namesToDisplayNames,
      state
    )
    {
      if (step && $(step).attr('id') != '')
      {
        $(row).attr('id', $(step).attr('id'));
      }

      $('input[name="user"]', row).each
      (
        function()
        {
          setAutoCompleteOnInput($(this), displayNames);

          if (step)
          {
            var assignedUser =
              $.pincette.util.dom.getText
              (
                $.pincette.util.dom.selectFirstElement(step, 'assigned-user')
              );

            if (assignedUser)
            {
              $(this).val(namesToDisplayNames[assignedUser]);
            }
          }
        }
      );

      $('select[name="status"]', row).each
      (
        function()
        {
          $('option', $(this)).remove();

          if (state.options.fields['re_status'])
          {
            var values = state.options.fields['re_status'].values;

            if (values)
            {
              for (var i = 0; i < values.length; ++i)
              {
                var option = $('<option></option>');

                option.attr('label', values[i]);
                option.attr('value', values[i]);
                option.text(values[i]);
                $(this).append(option);
              }
            }
          }

          if (step)
          {
            var status =
              $.pincette.util.dom.
                getText($.pincette.util.dom.selectFirstElement(step, 'status'));

            if (status)
            {
              $(this).val(status);
            }
          }
        }
      );
    }



    function isAuthorizedCommand(command, response, parentFolder)
    {
      var result =
        (!command.privileges && !command.parentPrivileges) ||
          (
            command.privileges && response.privileges &&
              $.pincette.webdav.
                hasAllPrivileges(response.privileges, command.privileges)
          ) ||
          (
            command.parentPrivileges && parentFolder &&
              parentFolder.privileges &&
              $.pincette.webdav.hasAllPrivileges
              (
                parentFolder.privileges,
                command.parentPrivileges
              )
          );

      return result;
    }



    function isChecked(fullTextName, state)
    {
      var result =
        $('input[name="' + fullTextName + '"]', state.simpleSearch).
          add('input[name="' + fullTextName + '"]', state.advancedSearch).
          prop('checked');

      return result;
    }



    function isDate(field, state)
    {
      var f = state.options.fields[field];

      return f && f.type == 'date';
    }



    function isSelectedPrincipal(element, label)
    {
      var result = $('option[label="' + label + '"]', element).length > 0;

      return result;
    }



    function isValueSet(control)
    {
      var value = $.pincette.util.ui.getFieldValue(control);

      return value != null && value != '' && value.trim().length > 0;
    }



    function link(response, state)
    {
      state.options.urlPicker.pincette
      (
        'pickUrl',
        {
          rootUrl: state.options.urlPickerRootUrl,
          sortDocuments: state.options.sortDocuments,
          sortFolders: state.options.sortFolders,
          documentsOnly: false,
          onChoose:
            function(resp)
            {
              state.dav.bind
              (
                resp.href,
                response.href + $.pincette.util.getLastPathSegment(resp.href) +
                  (resp.isCollection ? '/' : ''),
                function(){refresh(response.href, state);}
              );
            }
        }
      );
    }



    function loadAllMetaData(state)
    {
      for (var field in state.options.fields)
      {
        if (state.options.fields[field].load)
        {
          loadMetaData(field, state);
        }
      }
    }



    function loadFolder(url, usingHistory, state, done)
    {
      if (!url)
      {
        return;
      }

      var documents = [];
      var folders = [];
      var metaBrowse = url.indexOf('/meta/browse/');
      var parentFolder;
      var segments;

      if (metaBrowse != -1)
      {
        segments = $.pincette.util.getPathSegments(url.substring(metaBrowse));
      }

      if (state.searchDoneAction)
      {
        state.searchDoneAction.css('visibility', 'hidden');
      }

      state.count.empty();

      state.dav.propfind
      (
        url,
        getStandardProperties(state),
        '1',
        true,
        function(response)
        {
          if
          (
            (
              !state.options.filter					    ||
              state.options.filter(response)
            )								    &&
            (
              state.options.showUnreadable				    ||
              (
                response.privileges					    &&
                $.pincette.webdav.hasPrivilege(response.privileges, 'read')
              )
            )
          )
          {
            if (response.isCollection)
            {
              if
              (
                $.pincette.util.url.getPath(response.href) ==
                  $.pincette.util.url.getPath(url)
              )
              {
                parentFolder = response;
              }
              else
              {
                if
                (
                  !segments					  	||
                  !state.options.limitMetaFolders		  	||
                  segments.length % 2 != 0			  	||
                    // The results are values.
                  canShowProperty
                  (
                    $.pincette.util.getLastPathSegment(response.href),
                    state
                  )
                )
                {
                  folders.push(response);
                }
              }
            }
            else
            {
              if (response.meta && !response.meta.dc_title)
              {
                response.meta.dc_title = [response.name];
              }

              documents.push(response);
            }
          }
        },
        null,
        function()
        {
          state.count.append
          (
            state.options.loadLimit &&
              state.options.loadLimit < documents.length ?
              (state.options.loadLimit + '/' + documents.length) :
              documents.length
          );

          sortResponses
          (
            documents,
            state.lastSortField && state.lastSortField != 'normal' ?
              [state.lastSortField] : state.options.sortDocuments,
            state
          );

          if
          (
            state.options.loadLimit			&&
            state.options.loadLimit < documents.length
          )
          {
            documents = documents.slice(0, state.options.loadLimit);
          }

          if (folders.length > 0)
          {
            sortResponses(folders, state.options.sortFolders, state);
            state.folders.show();

            displayResultSet
            (
              folders,
              state.folders,
              state.options.rootUrl,
              function(response){loadFolder(response.href, false, state);},
              state,
              parentFolder,
              false
            );
          }
          else
          {
            state.folders.hide();
          }

          displayResultSet
          (
            documents,
            state.documents,
            state.options.rootUrl,
            state.options.openDocument,
            state,
            parentFolder,
            false
          );

          displayPath(url, state);
          state.options.searchUrl = url;
          state.currentUrl = url;

          if (parentFolder)
          {
            createResponseMenu(state.navigation, parentFolder, {}, state);
          }

          if (!usingHistory)
          {
            adjustHistory(url, state);
          }

          if (state.options.afterLoadFolder && parentFolder)
          {
            state.options.afterLoadFolder(parentFolder, state);
          }

          if (done)
          {
            done(folders, documents);
          }
        }
      );
    }



    function loadMetaData(field, state)
    {
      if (!state.options.fields[field].values)
      {
        var label =
          $.pincette.util.dc.getLabel(field, $.pincette.util.getLocale());

        state.options.fields[field].values = [];
        state.options.fields[field].load =
          asCollection
          (
            $.pincette.util.url.resolveUrl
            (
              $.pincette.util.url.getRootUrl(location.href),
              state.options.fields[field].load
            )
          );

        state.dav.propfind
        (
          state.options.fields[field].load +
            encodeURIComponent(label ? label : field) + '/',
          [{namespaceURI: $.pincette.webdav.dav, localName: 'displayname'}],
          '1',
          false,
          function(response)
          {
            state.options.fields[field].values.push(response.displayname);
          },
          null,
          function()
          {
            setAutoComplete
            (
              $('input[name="' + field + '"]', state.advancedSearch),
              state
            );
          }
        );
      }
    }



    function loading(state)
    {
      assignToState(state, 'loading', this);
    }



    function meAction(state)
    {
      assignToState(state, 'meAction', this).click
      (
        function()
        {
          state.dav.getCurrentUser
          (
            state.options.rootUrl,
            function(response)
            {
              editUser(response.href, state);
            }
          );
        }
      );
    }



    function messageBox(state)
    {
      assignToState(state, 'messageBox', this);
    }



    function metaData(state)
    {
      if (assignToState(state, 'metaData', this))
      {
        state.mandatoryMetaData = getMandatoryMetaData(state);
      }
    }



    function metaDataAddValue(state)
    {
      assignToState(state, 'metaDataAddValue', this);
    }



    function metaDataAddValueAction(response, state)
    {
      function fillValues(element, values)
      {
        element.empty();

        for (var i = 0; i < values.length; ++i)
        {
          var div = $('<div></div>');

          div.text(values[i]);
          element.append(div);
        }
      }

      var copy = state.metaDataAddValue.clone();
      var values = new Array();

      $('input[name="new"]', copy).keyup
      (
        function()
        {
          if (isValueSet($(this)))
          {
            var value = $(this).val().trim();

            if
            (
              (
                values.length <= 100	&&
                value.length >= 1
              )				||
              (
                values.length <= 1000	&&
                value.length >= 2
              )				||
              value.length >= 3
            )
            {
              fillValues
              (
                $('.values', copy),
                $.pincette.util.
                  sortCaseInsensitive(selectMatchingValues(values, value))
              );
            }
            else
            {
              $('.values', copy).empty();
            }
          }
        }
      );

      state.dav.propfind
      (
        response.href,
        [{namespaceURI: $.pincette.webdav.dav, localName: 'displayname'}],
        '1',
        false,
        function(response)
        {
          values.push
          (
            decodeURIComponent
            (
              $.pincette.util.getLastPathSegment(response.href)
            )
          );
        },
        null,
        function()
        {
          $.pincette.util.ui.dialogBox
          (
            null,
            {
              onOK:
                function(element, userClose, reEnableDialogBox)
                {
                  var input = $('input[name="new"]', copy);

                  if (isValueSet(input))
                  {
                    state.dav.mkcol
                    (
                      response.href + encodeURIComponent(input.val().trim()) +
                        '/',
                      function()
                      {
                        userClose();

                        if (response.href == getOpenFolder(state))
                        {
                          refresh(response.href, state);
                        }
                      },
                      function(status, text, method, headers)
                      {
                        reEnableDialogBox();
                        state.dav.
                          getDefaultFailure()(status, text, method, headers);
                      }
                    );
                  }
                }
            },
            copy
          );
        }
      );
    }



    function methodToClass(method)
    {
      var result = 'pincette-';

      for (i = 0; i < method.length; ++i)
      {
        result +=
          method.charAt(i) >= 'A' && method.charAt(i) <= 'Z' ?
            ('-' + method.charAt(i).toLowerCase()) : method.charAt(i);
      }

      return result;
    }



    function mkcol(response, state)
    {
      $.pincette.util.ui.getName
      (
        null,
        function(value, userClose)
        {
          state.dav.mkcol
          (
            response.href + encodeURIComponent(value) + '/',
            function()
            {
              userClose();
              refresh(response.href, state);
            },
            function(status, text, method, headers)
            {
              checkExists(status, text, method, headers, state);
            }
          );
        },
        state.nameBox.clone()
      );
    }



    function movePrincipal(element, selector, oneClass, otherClass)
    {
      var newParent =
        $(
          (
            element.parents('.' + oneClass).length > 0 ?
              ('.' + otherClass) : ('.' + oneClass)
          ) + ' .principals',
          selector
        );

      element.detach();
      newParent.append(element);

      $.pincette.util.ui.setListIndex
      (
        $.pincette.util.jq.sortContents
        (
          $('.principals', selector),
          sortPrincipalElementCriterion
        )
      );
    }



    function nameBox(state)
    {
      assignToState(state, 'nameBox', this);
    }



    function navigation(state)
    {
      assignToState(state, 'navigation', this);
    }



    function newDocument(state)
    {
      assignToState(state, 'newDocument', this);
    }



    function newDocumentAction(response, state)
    {
      var box = state.newDocument.clone();
      var parent = response.href;

      populateMimeTypes(box, state);

      $.pincette.util.ui.dialogBox
      (
        null,
        {
          onOK:
            function(element, userClose, reEnableDialogBox)
            {
              var name = $('input[name="name"]', box);
              var type = $('select[name="type"]', box);

              if (isValueSet(name) && isValueSet(type))
              {
                var extension =
                  '.' + $.pincette.util.getExtensionFromMimeType(type.val());
                var url =
                  response.href + encodeURIComponent(name.val().trim()) +
                    (
                      $.pincette.util.getExtension(name.val().trim()) != null ?
                        '' : extension
                    );

                state.dav.exists
                (
                  url,
                  function(response)
                  {
                    if (response)
                    {
                      $.pincette.util.ui.messageBox
                      (
                        getResource("err-exists"),
                        null,
                        null,
                        state.messageBox.clone()
                      );

                      reEnableDialogBox();
                    }
                    else
                    {
                      state.dav.create
                      (
                        url,
                        type.val(),
                        function()
                        {
                          userClose();
 
                          refresh
                          (
                            parent,
                            state,
                            function(folders, documents)
                            {
                              for (var i = 0; i < documents.length; ++i)
                              {
                                if (documents[i].href == url)
                                {
                                  editDocument(documents[i], state);
                                  break;
                                }
                              }
                            }
                          );
                        }
                      );
                    }
                  }
                );
              }
            }
        },
        box
      );
    }



    function newUser(state)
    {
      assignToState(state, 'newUser', this);
    }



    function newWorkflow(response, state)
    {
      var parent = response.href;

      $.pincette.util.ui.getName
      (
        null,
        function(value, userClose)
        {
          var url =
            response.href + encodeURIComponent(value) +
              ($.pincette.util.endsWith(value, '.pwfl') ? '' : '.pwfl');

          state.dav.exists
          (
            url,
            function(response)
            {
              if (response)
              {
                $.pincette.util.ui.messageBox
                (
                  getResource("err-exists"),
                  null,
                  null,
                  state.messageBox.clone()
                );
              }
              else
              {
                state.dav.create
                (
                  url,
                  'application/vnd.pincette.workflow+xml',
                  function()
                  {
                    userClose();

                    refresh
                    (
                      parent,
                      state,
                      function(folders, documents)
                      {
                        for (var i = 0; i < documents.length; ++i)
                        {
                          if (documents[i].href == url)
                          {
                            editWorkflow(documents[i], state);
                            break;
                          }
                        }
                      }
                    );
                  }
                );
              }
            }
          );
        },
        state.nameBox.clone()
      );
    }



    function nextWorkflow(response, state)
    {
      stepWorkflow(response, 're:next', null, state);
    }



    function notSetMetaData(element)
    {
      var result =
        $(
          'input.pincette-mandatory,textarea.pincette-mandatory,' +
            'select.pincette-mandatory', element
        ).filter(function(){return !isValueSet($(this));}).length > 0;

      return result;
    }



    function openDocument(response, state)
    {
      if ($.pincette.util.inArray(response.resourceTypes, 'document'))
      {
        window.open
        (
          (response.stableHref ? response.stableHref : response.href) +
            (
              state.options.defaultMediaType ?
                ('?type=' + encodeURIComponent(state.options.defaultMediaType))
                : ''
            )
        );
      }
    }



    function openForm(state)
    {
      if (state.showFormAction.hasClass('closed'))
      {
        state.forms.show();
        setResultHeight(state);
        state.showFormAction.removeClass('closed');
      }
    }



    function openHelp(options)
    {
      if (options.help)
      {
        window.open(options.help.url);
      }
    }



    function pastDue(response)
    {
      var value = $.pincette.webdav.getMetaDataValue(response, 're_dueDate');
      var result =
        value &&
          (
            $.pincette.util.parseISO8601(value).getTime() <
            new Date().getTime()
          );

      return result;
    }



    function paste(response, state)
    {
      var source =
        state.copyClipboard ? state.copyClipboard : state.cutClipboard;
      var destination =
        !response.isCollection ?
          response.href :
          (
            response.href + $.pincette.util.getLastPathSegment(source.href) +
              (source.isCollection ? '/' : '')
          );
      var messages = '';

      function reload()
      {
        refreshOrGoto($.pincette.util.getParent(destination), state);
      }

      function report(href, status, method, error, responseDescription)
      {
        messages +=
          reportMultistatus(href, status, method, error, responseDescription);
      }

      function done()
      {
        if (messages.length > 0)
        {
          $.pincette.util.ui.messageBox
          (
            messages,
            null,
            null,
            state.messageBox.clone()
          )
        }
        else
        {
          reload();
        }
      }

      if (state.copyClipboard)
      {
        state.dav.
          copy(source.href, destination, reload, null, null, report, done);
      }
      else
      {
        state.dav.
          move(source.href, destination, reload, null, null, report, done);
      }
    }



    function path(state)
    {
      assignToState(state, 'path', this);
    }



    function periodBox(state)
    {
      assignToState(state, 'periodBox', this);
    }



    function pickUrl(options)
    {
      var closeDialog;
      var element;
      var result;

      function choose()
      {
        if (result && options.onChoose)
        {
          options.onChoose(result);
        }
      }

      function selectItem(response, state, item)
      {
        if (state.details)
        {
          $('.pincette-show-details-action', item).pincette
          (
            'showDetailsAction',
            state.options,
            state,
            response
          );

          $('.pincette-show-details-action', item).
            dblclick(function(e){e.stopPropagation();});
        }

        item.click
        (
          function()
          {
            item.siblings().removeClass('selected');
            $('.pincette-path .pincette-path-segment', element).
              removeClass('selected');
            item.addClass('selected');
            result = response;

            if (!closeDialog)
            {
              choose();
            }
          }
        );

        item.dblclick
        (
          function()
          {
            result = response;
            choose();

            if (closeDialog)
            {
              closeDialog();
            }
          }
        );
      }

      options =
        $.extend
        (
          {
            afterLoadFolder:
              function(response)
              {
                $('body').data('pincette-last-url', response.href);

                if (!options.documentsOnly)
                {
                  var last =
                    $('.pincette-path .pincette-path-segment', element).last();

                  last.addClass('selected');
                  result = response;

                  last.click
                  (
                    function()
                    {
                      $('.pincette-documents > div', element).
                        removeClass('selected');
                      last.addClass('selected');
                      result = response;

                      if (!closeDialog)
                      {
                        choose();
                      }
                    }
                  );

                  last.dblclick
                  (
                    function()
                    {
                      result = response;
                      choose();

                      if (closeDialog)
                      {
                        closeDialog();
                      }
                    }
                  );
                }
              },
            dialog: true,
            displayResultEntry:
              function(response, parent, state, item)
              {
                if (item)
                {
                  parent.append(item);
                  item.show();

                  $.pincette.webdav.ui.replaceData
                  (
                    item,
                    response,
                    state.options.fields,
                    state.dav
                  );

                  if (response.isCollection)
                  {
                    item.click
                    (
                      function(){loadFolder(response.href, false, state);}
                    );
                  }
                  else
                  {
                    selectItem(response, state, item);
                  }
                }
              },
            documentsOnly: true,
            initialUrl: $('body').data('pincette-last-url'),
            useUrlParameters: false,
            annexesInBackground: false
          },
          options
        );

      if (options.dialog)
      {
        element = $(this).clone();
        closeDialog =
          $.pincette.util.ui.dialogBox
          (
            null,
            {
              onOK:
                function()
                {
                  choose();

                  return true;
                }
            },
            element.pincette('init', options)
          );
      }
      else
      {
        element = $(this);
        element.pincette('init', options);
      }
    }



    function plotEvents(events, map)
    {
      var data = new Array();
      var options =
        {
          axes: {xaxis: {}, yaxis: {min: 0}},
          grid: {shadow: false},
          seriesDefaults: {shadow: false, markerOptions: {shadow: false}}
        };

      for (var i = 0; i < events.length; ++i)
      {
        data.push
        (
          [
            map.x.type == 'string' ?
              $.pincette.util.getLastPathSegment(events[i][map.x.field]) :
              events[i][map.x.field],
            events[i][map.y.field]
          ]
        );
      }

      if (map.x.type == 'date' || map.x.type == 'time')
      {
        options.axes.xaxis.renderer = $.jqplot.DateAxisRenderer;

        options.axes.xaxis.tickOptions =
          {
            formatString:
              map.x.type == 'date' ?
                $.pincette.util.getShortDateFormat().
                  replace(/dd/, '%d').
                  replace(/d/, '%#d').
                  replace(/mm/, '%m').
                  replace(/m/, '%#m').
                  replace(/yy/, '%Y').
                  replace(/y/, '%y') :
                $.pincette.util.getTimeFormat().
                  replace(/HH/, '%H').
                  replace(/hh/, '%I').
                  replace(/mm/, '%M').
                  replace(/ss/, '%S').
                  replace(/a\.m\.\|p\.m\./, '%p')
          };
      }
      else
      {
        if (map.x.type = 'string')
        {
          options.seriesDefaults.renderer = $.jqplot.BarRenderer;
          options.axes.xaxis =
            {
              renderer: $.jqplot.CategoryAxisRenderer,
              tickRenderer: $.jqplot.CanvasAxisTickRenderer,
              tickOptions: {angle: -30}
            };
        }
      }

      $.jqplot('pincette-chart-id', [data], options);
    }



    function populateGroupField(element, state)
    {
      var group = $('select[name="group"]', element);

      if (group.length > 0)
      {
        state.dav.getGroups
        (
          state.options.rootUrl,
          state.options.contextPath,
          null,
          function(groups)
          {
            $.pincette.util.sortCaseInsensitive(groups);

            group.each
            (
              function()
              {
                for (var i = 0; i < groups.length; ++i)
                {
                  if
                  (
                    groups[i].response.privileges			  &&
                    $.pincette.webdav.
                      hasPrivilege(groups[i].response.privileges, 'bind')
                  )
                  {
                    var option = $('<option></option>');

                    option.attr('label', groups[i].getDisplayname());
                    option.attr('value', groups[i].getValue());
                    option.text(groups[i].getDisplayname());
                    $(this).append(option);
                  }
                }
              }
            );
          },
          false,
          true
        );
      }
    }



    function populateMimeTypes(box, state)
    {
      var labelled = [];
      var types = $('select[name="type"]', box);

      for (var i in state.options.editors)
      {
        labelled.push({type: i, label: $.pincette.util.getMimeTypeLabel(i)});
      }

      labelled.sort
      (
        function(o1, o2)
        {
          var result =
            o1.label.toLocaleLowerCase().
              localeCompare(o2.label.toLocaleLowerCase());

          return result;
        }
      );

      for (var i = 0; i < labelled.length; ++i)
      {
        var option = $('<option></option>');

        option.attr('label', labelled[i].label);
        option.attr('value', labelled[i].type);
        option.text(labelled[i].label);
        types.append(option);
      }
    }



    function populateUsersField(element, fieldName, state, done)
    {
      var field = $('select[name="' + fieldName + '"]', element);

      if (field.length > 0)
      {
        state.dav.getUsers
        (
          state.options.rootUrl,
          state.options.contextPath,
          null,
          function(users)
          {
            $.pincette.util.sortCaseInsensitive(users);

            field.each
            (
              function()
              {
                for (var i = 0; i < users.length; ++i)
                {
                  var option = $('<option></option>');

                  option.attr('label', users[i].getDisplayname());
                  option.attr('value', users[i].getValue());
                  option.text(users[i].getDisplayname());
                  $(this).append(option);
                }
              }
            );

            if (done)
            {
              done(field, users);
            }
          }
        );
      }
    }



    function preload(state)
    {
      if (state.options.preload)
      {
        if
        (
          state.options.parameters					&&
          (
            preloadSearch(state.options.parameters, state)		||
            (
              state.options.parameters.query				&&
              state.options.parameters.query.length > 0			&&
              preloadSearch
              (
                $.pincette.util.url.extractParameters
                (
                  state.options.parameters.query[0].
                    replace(/\|/g, '&').replace(/\(/g, '=').
                    replace(/\)/g, ''),
                  false
                ),
                state
              )
            )
          )
        )
        {
          search(state);
        }
        else
        {
          if (state.options.initialUrl)
          {
            loadFolder
            (
              state.options.parameters && state.options.parameters.path ?
                $.pincette.util.url.resolveUrl
                (
                  state.options.initialUrl,
                  encodeURI(state.options.parameters.path[0])
                ) : state.options.initialUrl,
              false,
              state
            );
          }
        }
      }
    }



    function preloadSearch(parameters, state)
    {
      var loaded = false;

      for (var field in parameters)
      {
        var input =
          $('input[name="' + field + '"]', state.simpleSearch).
            add($('input[name="' + field + '"]', state.advancedSearch));

        input.each
        (
          function(index)
          {
            if (index < parameters[field].length)
            {
              $.pincette.util.ui.
                setFieldValue($(this), state.options.parameters[field][index]);
              loaded = true;
            }
          }
        );

        for (var i = input.length; i < parameters[field].length; ++i)
        {
          addFieldRow
          (
            input.last(),
            state,
            function(row, state)
            {
              initializeSearchRow(row, state);

              $('input', row).first().each
              (
                function()
                {
                  $.pincette.util.ui.
                    setFieldValue($(this), state.options.parameters[field][i]);
                  loaded = true;
                }
              );
            }
          );
        }
      }

      if (loaded)
      {
        clearContains(state);
      }

      return loaded;
    }



    function prepareDisplayResultSet(element)
    {
      var result = {};

      result.template = $('.template', element);
      element.scrollTop(0);

      if (result.template.length > 0)
      {
        result.parent = result.template.parent();
        result.template.hide();
      }
      else
      {
        result.parent = element;
      }
    
      result.parent.empty();

      return result;
    }



    function prepareOptions(options)
    {
      var pageRoot = $.pincette.util.url.getRootUrl(location.href);
      var allOptions =
        $.extend
        (
          {
            annexesInBackground: false,
            commands: [],
            displayResultEntry: displayResultEntry,
            openDocument: openDocument,
            preload: true,
            recentAgeInDays: 15,
            rootUrl: pageRoot,
            searchLimit: 100,
            searchResultMaxHeight: false,
            shareWithGroupsOnly: false,
            showHidden: false,
            showUnreadable: false,
            topIsPathParameter: false,
            urlPicker: $('.pincette-url-picker'),
            useUrlParameters: true
          },
          $.isArray(options) ?
            getOptionsFromPath(cascadeOptions(options)) : options
        );

      if (allOptions.rootUrl)
      {
        allOptions.rootUrl =
          $.pincette.util.url.resolveUrl(pageRoot, allOptions.rootUrl);
      }

      allOptions.fields =
        $.extend({getlastmodified: {type: 'time'}}, allOptions.fields);
      allOptions.parameters =
        allOptions.useUrlParameters ?
          $.pincette.util.url.getParameters(location.href) : {};

      if (allOptions.topIsPathParameter && allOptions.parameters.path)
      {
        allOptions.rootUrl = 
          $.pincette.util.url.resolveUrl
          (
            allOptions.rootUrl,
            encodeURI(allOptions.parameters.path[0])
          );
      }

      if (allOptions.rootUrl)
      {
        allOptions.rootUrl = asCollection(allOptions.rootUrl);
      }

      allOptions.searchUrl =
        allOptions.searchUrl ?
          asCollection
          (
            $.pincette.util.url.resolveUrl(pageRoot, allOptions.searchUrl)
          ) : allOptions.rootUrl;

      allOptions.initialUrl =
        allOptions.initialUrl ?
          asCollection
          (
            $.pincette.util.url.resolveUrl(pageRoot, allOptions.initialUrl)
          ) : allOptions.rootUrl;

      allOptions.urlPickerRootUrl =
        allOptions.urlPickerRootUrl ?
          asCollection
          (
            $.pincette.util.url.
              resolveUrl(pageRoot, allOptions.urlPickerRootUrl)
          ) : allOptions.rootUrl;

      if (allOptions.uploadUrl)
      {
        allOptions.uploadUrl =
          $.pincette.util.url.resolveUrl(pageRoot, allOptions.uploadUrl);
      }

      if (allOptions.passUrl)
      {
        allOptions.passUrl =
          $.pincette.util.url.resolveUrl(pageRoot, allOptions.passUrl);
      }

      if (!allOptions.sortDocuments || allOptions.sortDocuments.length == 0)
      {
        allOptions.sortDocuments = ['score', 'dc_title', 'name'];
      }

      if (!allOptions.sortFolders || allOptions.sortFolders.length == 0)
      {
        allOptions.sortFolders = ['name'];
      }

      if (allOptions.contextPath)
      {
        allOptions.contextPath =
          $.pincette.util.stripTrailingSlash(allOptions.contextPath);
      }

      return allOptions;
    }



    function prepareState(state, options)
    {
      if (!state.history)
      {
        state.options = options;
        state.activities = 0;
        state.history = new Array();
        state.historyPosition = 0;
        state.runningSearch = null;

        for (var method in initMethods)
        {
          state[method] = $();
        }

        state.dav =
          new $.pincette.webdav.Client
          (
            state.options.oauth,
            function()
            {
              $.pincette.util.ui.messageBox
              (
                getResource('err-expired'),
                null,
                function(){location.reload();},
                state.messageBox.clone()
              );
            }
          );

        state.dav.setDefaultFailure 
        (
          function(status, responseText, method, headers)
          {
            if (status != 0 && responseText != 'abort')
            {
              if (state.options.defaultFailure)
              {
                state.options.
                  defaultFailure(status, responseText, method, headers);
              }
              else
              {
                $.pincette.util.ui.messageBox
                (
                  status == 12019 ?
                    'Certificaat fout' :
                    $.pincette.webdav.ui.
                      getStatusMessage(status, method, responseText),
                  null,
                  null,
                  state.messageBox.clone()
                );
              }
            }
          }
        );

        state.dav.setContextPath(state.options.contextPath);
        state.dav.setStartActivity(function(){state.addActivity();});
        state.dav.setEndActivity(function(){state.removeActivity();});

        if (options.searchResultMaxHeight)
        {
          $(window).resize(function(){setResultHeight(state);});
        }

        if (state.options.builtInCommands)
        {
          var builtIn = new Array();

          for (var i = 0; i < state.options.builtInCommands.length; ++i)
          {
            var command = getCommand(state.options.builtInCommands[i]);

            if (command)
            {
              builtIn.push(command);
            }
          }

          state.options.commands = builtIn.concat(state.options.commands);
        }
        else
        {
          state.options.commands = commands.concat(state.options.commands);
        }

        checkAudit(state);
        loadAllMetaData(state);
      }

      return state;
    }



    function previewBox(state)
    {
      assignToState(state, 'previewBox', this);
    }



    function previousWorkflow(response, state)
    {
      stepWorkflow(response, 're:previous', null, state);
    }



    function propagateAcl(response, changes, state, done)
    {
      var reporter = state.reporter.clone();

      $.pincette.util.ui.modalWithClose(reporter);

      state.dav.propagateAcl
      (
        response,
        changes,
        function(response)
        {
          $.pincette.util.ui.addMessage
          (
            reporter,
            $.pincette.webdav.getPath(response.href, state.options.contextPath),
            false
          );
        },
        function(status, text, headers, method, url)
        {
          $.pincette.util.ui.addMessage
          (
            reporter,
            $.pincette.webdav.getPath(url, state.options.contextPath)
              + ': ' + text,
            true
          );

          return true;
        },
        done
      );
    }



    function recent(response, state)
    {
      executeSearch
      (
        response.href,
        [
          {
            field: 'getlastmodified',
            op: '>=',
            value:
              $.pincette.util.timestampISO8601
              (
                new Date
                (
                  new Date().getTime() -
                    state.options.recentAgeInDays * 24 * 60 * 60 * 1000
                )
              )
          }
        ],
        ['!getlastmodified'],
        state
      );
    }



    function refresh(folder, state, done)
    {
      var documentsTop = state.documents.scrollTop();
      var foldersTop = state.folders.scrollTop();

      loadFolder
      (
        folder,
        state.history[state.historyPosition] == folder,
        state,
        function(folders, documents)
        {
          state.documents.scrollTop(documentsTop);
          state.folders.scrollTop(foldersTop);

          if (done)
          {
            done(folders, documents);
          }
        }
      ); // Using the history corresponds to a refresh in this case.
    }



    function refreshOrGoto(folder, state)
    {
      if (folder.charAt(folder.length - 1) == '/')
      {
        if (folder == state.currentUrl)
        {
          refresh(folder, state);
        }
        else
        {
          loadFolder(folder, false, state);
        }
      }
    }



    function refreshSingle(response, state)
    {
      var displayElement = response.displayElement;

      if (displayElement && displayElement.item && displayElement.template)
      {
        state.dav.getProperties
        (
          response.href,
          getStandardProperties(state),
          function(response)
          {
            var copy = displayElement.template.clone().removeClass('template');

            displayElement.item.replaceWith(copy);
            response.displayElement = displayElement;
            response.displayElement.item = copy;

            state.options.displayResultEntry
            (
              response,
              displayElement.parentElement,
              state,
              copy
            );
          }
        );
      }
    }



    function relations(state)
    {
      assignToState(state, 'relations', this);
    }



    function relationsAction(response, relationName, filter, maxRows, state)
    {
      function edit(locks)
      {
        var count = 0;
        var notEdited = [];
        var relations = state.relations.clone();
        var template = $('.template', relations);
        var placeHolderText = $('.url', template).text();

        if (response.referringTo && response.referringTo.length > 0)
        {
          for (var i = 0; i < response.referringTo.length; ++i)
          {
            if (relationName == response.referringTo[i].name)
            {
              ++count;

              var item = template.clone();

              item.removeClass('template');

              initializeRelation
              (
                item,
                response.referringTo[i],
                filter,
                state,
                placeHolderText
              );

              template.parent().append(item);
            }
            else
            {
              notEdited.push(response.referringTo[i]);
            }
          }
        }

        if (count == 0)
        {
          template.removeClass('template');
          initializeRelation(template, null, filter, state, placeHolderText);
        }
        else
        {
          template.remove();
        }

        $.pincette.util.ui.editTableRows
        (
          relations,
          function(row)
          {
            initializeRelation(row, null, filter, state, placeHolderText);
          },
          null,
          function(row)
          {
            initializeRelation(row, null, filter, state, placeHolderText);
          },
          maxRows
        );

        $.pincette.util.ui.dialogBox
        (
          null,
          {
            onCancel:
              function()
              {
                if (locks)
                {
                  state.dav.unlock
                  (
                    response.href,
                    locks[0].token,
                    function(){refreshSingle(response, state);}
                  );
                }
              },
            onOK:
              function(element, userClose, reEnableDialogBox)
              {
                updateRelations
                (
                  element,
                  response,
                  relationName,
                  notEdited,
                  locks ? locks[0].token : null,
                  state,
                  userClose,
                  reEnableDialogBox
                );
              }
          },
          relations
        );
      }

      if (response.isCollection)
      {
        edit(null);
      }
      else
      {
        state.dav.lock(response.href, -1, edit);
      }
    }



    function relationsChanged(oldRelations, newRelations)
    {
      if
      (
        (
          !oldRelations					&&
          newRelations
        )						||
        (
          oldRelations					&&
          !newRelations
        )						||
        oldRelations.length != newRelations.length
      )
      {
        return true;
      }

      for (var i = 0; i < newRelations.length; ++i)
      {
        var found = false;

        for (var j = 0; !found && j < oldRelations.length; ++j)
        {
          if
          (
            newRelations[i].name == oldRelations[j].name	&&
            newRelations[i].href == oldRelations[j].href
          )
          {
            found = true;
          }
        }

        if (!found)
        {
          return true;
        }
      }

      return false;
    }



    function removeEmpty(values)
    {
      var result = new Array();

      for (var i = 0; i < values.length; ++i)
      {
        if (values[i] != '')
        {
          result.push(values[i]);
        }
      }

      return result;
    }



    function rename(response, state)
    {
      var oldName = $.pincette.util.getLastPathSegment(response.href);

      $.pincette.util.ui.getName
      (
        decodeURIComponent(oldName),
        function(value, userClose)
        {
          if (value != decodeURIComponent(oldName))
          {
            var newUrl =
              $.pincette.util.getParent(response.href) +
                encodeURIComponent(value);

            state.dav.move
            (
              response.href,
              newUrl,
              function()
              {
                userClose();
                response.href = newUrl;
                refreshSingle(response, state);
              }
            );
          }
        },
        state.nameBox.clone()
      );
    }



    function reportMultistatus(href, status, method, error, responseDescription)
    {
      var result =
        '<div class="pincette-multiline-message">' +
          $.pincette.util.url.getPath(href) + ': ' +
          $.pincette.webdav.ui.
            report(method, status, error, responseDescription) +
          '</div>';

      return result;
    }



    function reportProppatchErrors(errors)
    {
      var messages = '';

      for (var i = 0; i < errors.length; ++i)
      {
        messages +=
          '<div class="pincette-multiline-message">' + errors[i].message +
          '</div>';
      }

      return messages;
    }



    function reporter(state)
    {
      assignToState(state, 'reporter', this);
    }



    function resortDocuments(fields, state)
    {
      $.pincette.util.jq.sort
      (
        documentItems(state),
        function(i1, i2)
        {
          var result =
            sortResponsesCriterion
            (
              $(i1).data('pincette-response'),
              $(i2).data('pincette-response'),
              fields && fields.length == 1 && fields[0] == 'normal' ?
                state.options.sortDocuments : fields,
              state
            );

          return result;
        }
      );

      $.pincette.util.ui.setListIndex(state.documents);
    }



    function resourceClass(response)
    {
      if (response.isCollection)
      {
        return 'pincette-is-collection';
      }

      if (!response.responseTypes)
      {
        return 'pincette-is-document';
      }

      for (var i = 0; i < response.resourceTypes.length; ++i)
      {
        if (response.resourceTypes[i] != 'collection')
        {
          return 'pincette-is-' + response.resourceTypes[i];
        }
      }

      return null;
    }



    function retainLastRowWithName(row)
    {
      var name = $('input,select', row).attr('name');
      var result =
        name == $('input,select', row.prev()).attr('name') ||
          name == $('input,select', row.next()).attr('name');

      return result;
    }



    function search(state)
    {
      var expressions = generateExpression(state);

      if (expressions.length > 0)
      {
        executeSearch
        (
          state.options.searchUrl,
          expressions,
          null,
          state
        );
      }
    }



    function searchAction(state)
    {
      assignToState(state, 'searchAction', this).
        click(function(){search(state);});
      addEnterEvents(state, state.simpleSearch);
      addEnterEvents(state, state.advancedSearch);
    }



    function searchDoneAction(state)
    {
      assignToState(state, 'searchDoneAction', this).click
      (
        function()
        {
          refresh
          (
            state.history[state.historyPosition] ?
              state.history[state.historyPosition] : state.options.rootUrl,
            state
          );
        }
      );
    }



    function selectCommand(response, state, command)
    {
      var result =
        !command.filter ||
          (
            hasNeededResourceType(response, command) &&
              hasNeededContentType(response, command) &&
              hasNeededMethod(response, command) &&
              (
                !command.filter.filenamePattern ||
                  new RegExp(command.filter.filenamePattern).
                    test($.pincette.util.url.getPath(response.href))
              ) &&
              (
                !command.filter.action ||
                  command.filter.action(response, state, command)
              )
          );

      return result;
    }



    function selectMatchingValues(values, value)
    {
      if (value.trim().length == 0)
      {
        return values;
      }

      var result = new Array();

      value = value.replace(/\s+/g, '').toLowerCase();

      for (var i = 0; i < values.length; ++i)
      {
        if
        (
          values[i].replace(/\s+/g, '').toLowerCase().indexOf(value) != -1
        )
        {
          result.push(values[i]);
        }
      }

      return result;
    }



    function selectMostSpecific(paths, rootPath)
    {
      var result = null;

      for (var i = 0; i < paths.length; ++i)
      {
        if
        (
          $.pincette.util.startsWith(rootPath, paths[i])	&&
          (
            result == null					||
            paths[i].length > result.length
          )
        )
        {
          result = paths[i];
        }
      }

      return result;
    }



    function selectNonSharingAces(aces, privilege)
    {
      var result = new Array();

      for (var i = 0; i < aces.length; ++i)
      {
        if
        (
          aces[i].protected						||
          !aces[i].grant						||
          (
            aces[i].privilege.namespaceURI != $.pincette.webdav.dav	&&
            aces[i].privilege.namespaceURI != $.pincette.webdav.re
          )								||
          aces[i].privilege.namespaceURI != privilege.namespaceURI	||
          aces[i].privilege.localName != privilege.localName
        )
        {
          result.push(aces[i]);
        }
      }

      return result;
    }



    function selectPrincipals(state, response)
    {
      assignToState(state, 'selectPrincipals', this);
    }



    function selectPrincipalsAction(properties, expandGroups, state, done)
    {
      if
      (
        !state.selectPrincipals			||
        state.selectPrincipals.length == 0	||
        !done
      )
      {
        return;
      }

      var all = [];
      var copy = state.selectPrincipals.clone();

      state.dav.startChaining();

      state.dav.getGroups
      (
        state.options.rootUrl,
        state.options.contextPath,
        properties,
        function(groups)
        {
          all = all.concat(groups);
        },
        false,
        true
      );

      state.dav.getUsers
      (
        state.options.rootUrl,
        state.options.contextPath,
        properties,
        function(users)
        {
          all = all.concat(users);
        }
      );

      state.dav.runChain
      (
        function()
        {
          fillPrincipalSelector(copy, all, [], 'selected');

          $.pincette.util.ui.dialogBox
          (
            null,
            {
              onOK:
                function(element, userClose, reEnableDialogBox)
                {
                  if (expandGroups)
                  {
                    state.dav.expandGroups
                    (
                      getSelectedPrincipals(copy, 'selected'),
                      properties,
                      function(users)
                      {
                        done(users, userClose, reEnableDialogBox);
                      }
                    );
                  }
                  else
                  {
                    done
                    (
                      getSelectedPrincipals(copy, 'selected'),
                      userClose,
                      reEnableDialogBox
                    );
                  }
                },
              afterShow:
                function()
                {
                  $.pincette.util.ui.setListIndex($('.principals', copy));
                }
            },
            copy
          );
        }
      );
    }



    function selectWorkflowStep(response, steps, state, done)
    {
      var copy = state.workflowSteps.clone();
      var select = $('select[name="step"]', copy);

      for (var i = 0; i < steps.length; ++i)
      {
        var option = $('<option></option>');

        option.attr('label', steps[i].assignedUser + '/' + steps[i].status);
        option.text(steps[i].assignedUser + '/' + steps[i].status);

        option.attr
        (
          'value',
          steps[i].id + '#' + steps[i].assignedUser + '#' + steps[i].status
        );

        select.append(option);
      }

      if
      (
        response.meta.re_workflowStepId	&&
        response.meta.re_assignedUser	&&
        response.meta.re_status
      )
      {
        select.val
        (
          response.meta.re_workflowStepId[0] + '#' +
            response.meta.re_assignedUser[0] + '#' + response.meta.re_status[0]
        );
      }

      $.pincette.util.ui.dialogBox
      (
        null,
        $.extend
        (
          {
            onOK:
              function(element, userClose)
              {
                userClose();

                if (isValueSet(select))
                {
                  var a = select.val().split('#');

                  if (a[0] != response.meta.re_workflowStepId)
                  {
                    done({id: a[0], assignedUser: a[1], status: a[2]});
                  }
                }
              }
          },
          null
        ),
        copy
      );
    }



    function selectWorkflowVersions(responses)
    {
      var result = [];

      for (var i = 0; i < responses.length; ++i)
      {
        if (responses[i].labels)
        {
          var found = false;

          for (var j = 0; j < responses[i].labels.length && !found; ++j)
          {
            if ($.pincette.util.startsWith(responses[i].labels[j], 'WORKFLOW_'))
            {
              found = true;
            }
          }

          if (found)
          {
            result.push(responses[i]);
          }
        }
      }

      return result;
    }



    function sendMail(response, state)
    {
      selectPrincipalsAction
      (
        [{namespaceURI: $.pincette.webdav.re, localName: 'email'}],
        true,
        state,
        function(principals, close)
        {
          close();

          var emailAddresses = [];

          for (var i = 0; i < principals.length; ++i)
          {
            if (principals[i].response.email)
            {
              emailAddresses.push(principals[i].response.email);
            }
          }

          $.pincette.util.openLocally
          (
            'mailto:' + emailAddresses.join(',') + '?subject=' +
              encodeURIComponent
              (
                response.meta && response.meta.dc_title ?
                  response.meta.dc_title[0] : response.name
              ) + '&body=' +
              $.pincette.util.url.resolveUrl
              (
                $.pincette.util.url.getRootUrl(location.href),
                $.pincette.util.url.encodeUriPath
                  // Double encoding because this is in a URL parameter.
                (
                  response.stableHref ? response.stableHref : response.href
                )
              )
          );
        }
      );
    }



    function sendPass(response, state)
    {
      var now = new Date();

      $.pincette.util.ui.getDate
      (
        now,
        function(date)
        {
          if (!$.pincette.util.isBeforeDate(date, now))
          {
            date.setHours(23);
            date.setMinutes(59);
            date.setSeconds(59);

            state.dav.get
            (
              state.options.passUrl + '?url=' +
                encodeURIComponent
                (
                  response.stableHref ? response.stableHref : response.href
                ) + '&duration=' + String(date.getTime() - now.getTime()),
              function(status, data, headers)
              {
                $.pincette.util.openLocally
                (
                  'mailto:?subject=' +
                    encodeURIComponent
                    (
                      response.meta && response.meta.dc_title ?
                        response.meta.dc_title[0] : response.name
                    ) + '&body=' +
                    encodeURIComponent
                    (
                      $.pincette.util.url.resolveUrl
                      (
                        state.options.rootUrl,
                        $.pincette.util.dom.getText
                        (
                          $.pincette.util.dom.selectFirstElement
                          (
                            $.parseXML(data).documentElement,
                            'url'
                          )
                        )
                      )
                    )
                );
              }
            );
          }
        },
        state.dateBox.clone(),
        now.getFullYear(),
        now.getFullYear() + 1
      );
    }



    function setAutoComplete(input, state)
    {
      var field = state.options.fields[input.attr('name')];

      if (field && field.values)
      {
        setAutoCompleteOnInput(input, field.values);
      }
    }



    function setAutoCompleteOnInput(input, values)
    {
      input.autocomplete
      (
        {
          delay: 0,
          minLength: values.length <= 100 ? 1 : (values.length <= 1000 ? 2 : 3),
          source:
            function(request, response)
            {
              var selected = selectMatchingValues(values, request.term);

              if (selected.length == 0)
              {
                input.addClass('pincette-bad-value');
              }
              else
              {
                input.removeClass('pincette-bad-value');
              }

              response(selected);
            }
        }
      );

      input.keyup // Make sure backspace is also caught.
      (
        function()
        {
          if (!isValueSet($(this)))
          {
            $(this).removeClass('pincette-bad-value');
          }
        }
      );
    }



    function setChart(element, events, map, state)
    {
      if
      (
        state.chart			&&
        state.chart.length > 0		&&
        events.length > 0		&&
        (
          map.x.type == 'integer'	||
          map.x.type == 'date'		||
          map.x.type == 'string'	||
          map.x.type == 'time'
        )				&&
        (
          map.y.type == 'integer'	||
          map.y.type == 'duration'
        )
      )
      {
        $('.chart', element).click
        (
          function()
          {
            var copy = state.chart.clone();

            $('.chart', copy).attr('id', 'pincette-chart-id');

            $('.chart', copy).width
            (
              (
                events.length < 10 || map.x.type == 'string' ?
                  3 : (events.length < 20 ? 2 : 1)
              ) * events.length + 'em'
            );

            var close =
              $.pincette.util.ui.modal
              (
                copy,
                {
                  onShow:
                    function(container, overlay, box)
                    {
                      overlay.show();
                      container.show();
                      plotEvents(events, map);

                      if ($('.pincette-title-bar', copy).length == 0)
                      {
                        $.pincette.util.ui.addClose(copy, close);
                      }
                    }
                }
              );
          }
        );
      }
    }



    function setEventSubject(response, fields)
    {
      if ($.pincette.webdav.isPrincipal(response))
      {
        fields.user = response.href;
      }
      else
      {
        fields.path =
          $.pincette.util.url.getPath(response.href) +
            (response.isCollection ? '*' : '');
      }
    }



    function setFormWidgets(element, state)
    {
      if (state.options.fields)
      {
        $('input', element).each
        (
          function()
          {
            var input = state.options.fields[$(this).attr('name')];

            if (input && input.type == 'date')
            {
              $(this).removeClass('hasDatepicker').removeAttr('id').
                // If cloned.
                datepicker(getDatepickerRegional()).
                datepicker
                (
                  'option',
                  'dateFormat',
                  $.pincette.util.getShortDateFormat()
                );
            }
          }
        );
      }
    }



    function setMetaData(element, response, state)
    {
      var field = element.attr('name');
      var row = $.pincette.util.getClosestRow(element);

      initializeMetaDataRow(row, state);

      if (response.meta[field] && response.meta[field].length > 0)
      {
        element.val(displayField(field, response.meta[field][0], state));

        if (response.meta[field].length > 1)
        {
          for (var i = 1; i < response.meta[field].length; ++i)
          {
            addFieldRow
            (
              element,
              state,
              function(row, state)
              {
                initializeMetaDataRow(row, state);
                setPincetteFields(row);

                $('*[name="' + field + '"]', row).val
                (
                  displayField(name, response.meta[field][i], state)
                );
              }
            );
          }
        }
      }

      setPincetteFields(row);
    }



    function setMetaDataFixed(response, state)
    {
      function dialogBox()
      {
        var copy =
          $.pincette.webdav.isPrincipal(response) ?
            state.userMetaData.clone() : state.metaData.clone();

        $.pincette.util.ui.editTableRows
        (
          copy,
          function(row)
          {
            initializeMetaDataRow(row, state);
            setPincetteFields(row);
          },
          retainLastRowWithName
        );

        $.pincette.util.ui.dialogBox(null, options, copy);
      }

      var options =
        {
          onOK:
            function(element, userClose, reEnableDialogBox)
            {
              updateMetaData
              (
                element,
                response,
                receivedLocks ? receivedLocks[0].token : null,
                state,
                userClose,
                reEnableDialogBox
              );
            },
          onShow:
            function(element)
            {
              // Delay until here, because we need the display types in
              // order to detect rows.
              $('input,textarea,select', element).
                each(function(){setMetaData($(this), response, state);});
            },
          validate:
            function(element){return validateMetaData(element, state);}
        };
      var receivedLocks;

      if ($.pincette.webdav.isPrincipal(response))
      {
        dialogBox();
      }
      else
      {
        options.onCancel =
          function()
          {
            if (receivedLocks)
            {
              state.dav.unlock
              (
                response.href,
                receivedLocks[0].token,
                function(){refreshSingle(response, state);}
              );
            }
          };

        state.dav.lock
        (
          response.href,
          -1,
          function(locks)
          {
            receivedLocks = locks;
            dialogBox();
          }
        );
      }
    }



    function setPincetteFields(element)
    {
      $('input,textarea', element).not(':hidden').
        each(function(){$.pincette.util.ui.field($(this))});
    }



    function setReloadActivity(element, state, from, to, reload)
    {
      if (state.periodBox && state.periodBox.length > 0)
      {
        $('.period', element).click
        (
          function()
          {
            $.pincette.util.ui.getPeriod
            (
              from,
              to,
              reload,
              state.periodBox.clone(),
              2007,
              new Date().getFullYear()
            );
          }
        );
      }
    }



    function setResultHeight(state)
    {
      if (state.options.searchResultMaxHeight)
      {
        if (state.documents.length > 0)
        {
          state.documents.
            height($(window).height() - state.documents.offset().top - 15);
        }

        if (state.folders.length > 0)
        {
          state.folders.
            height($(window).height() - state.folders.offset().top - 15);
        }
      }
    }



    function setUserFieldValue(select, value)
    {
      if (value.indexOf('/') != -1)
      {
        select.val(value);
      }
      else
      {
        $('option', select).each
        (
          function()
          {
            if
            (
              value ==
                decodeURIComponent
                (
                  $.pincette.util.getLastPathSegment($(this).attr('value'))
                )
            )
            {
              select.val($(this).attr('value'));

              return false;
            }
          }
        );
      }
    }



    function sharing(state, response)
    {
      assignToState(state, 'sharing', this);
    }



    function sharingBindCommand(response, state)
    {
      sharingCommand
      (
        response,
        state,
        {namespaceURI: $.pincette.webdav.dav, localName: 'unbind'}
      );
    }



    function sharingCommand(response, state, privilege)
    {
      var all = [];
      var copy = state.sharing.clone();
      var currentAces;
      var granted = [];

      state.dav.startChaining();

      state.dav.getAcl
      (
        response.href,
        function(aces)
        {
          if (aces)
          {
            currentAces = aces;
            getGrantedPrincipals(aces, copy, privilege, state, granted);
          }
        },
        function(href, status, method, error, responseDescription)
        {
          $.pincette.util.ui.messageBox
          (
            reportMultistatus(href, status, method, error, responseDescription),
            null,
            null,
            state.messageBox.clone()
          );
        }
      );

      state.dav.getGroups
      (
        state.options.rootUrl,
        state.options.contextPath,
        null,
        function(groups)
        {
          all = all.concat(groups);
        },
        true,
        false
      );

      if (!state.options.shareWithGroupsOnly)
      {
        state.dav.getUsers
        (
          state.options.rootUrl,
          state.options.contextPath,
          null,
          function(users)
          {
            all = all.concat(users);
          }
        );
      }

      state.dav.runChain
      (
        function()
        {
          if (!currentAces)
          {
            return;
          }

          fillPrincipalSelector(copy, all, granted, 'granted');

          $.pincette.util.ui.dialogBox
          (
            null,
            {
              onOK:
                function(element, userClose, reEnableDialogBox)
                {
                  var newGranted = getSelectedPrincipals(copy, 'granted');
                  var newAces =
                    selectNonSharingAces(currentAces, privilege).concat
                    (
                      $.pincette.webdav.createAces
                      (
                        $.pincette.webdav.getPrincipalValues(newGranted),
                        privilege
                      )
                    );
                  var changes = $.pincette.util.changed(currentAces, newAces);

                  if (changes.added.length > 0 || changes.deleted.length > 0)
                  {
                    if
                    (
                      $.pincette.webdav.
                        isPureCollection(response, state.options.contextPath) &&
                      $('input[name="propagate"]:checked', copy).length == 1
                    )
                    {
                      userClose();
                      propagateAcl(response, changes, state, reEnableDialogBox);
                    }
                    else
                    {
                      if (!$.pincette.util.arraysEqual(granted, newGranted))
                      {
                        state.dav.acl
                        (
                          response.href,
                          newAces,
                          function(){userClose();},
                          function(status, text, method, headers)
                          {
                            reEnableDialogBox();

                            state.dav.getDefaultFailure()
                            (
                              status,
                              text,
                              method,
                              headers
                            );
                          }
                        );
                      }
                    }
                  }
                  else
                  {
                    userClose();
                  }
                },
              afterShow:
                function()
                {
                  $.pincette.util.ui.setListIndex($('.principals', copy));
                }
            },
            copy
          );
        }
      );
    }



    function sharingReadCommand(response, state)
    {
      sharingCommand
      (
        response,
        state,
        {namespaceURI: $.pincette.webdav.dav, localName: 'read'}
      );
    }



    function sharingReadVersionHistoryCommand(response, state)
    {
      sharingCommand
      (
        response,
        state,
        {namespaceURI: $.pincette.webdav.re, localName: 'read-version-history'}
      );
    }



    function sharingUnbindCommand(response, state)
    {
      sharingCommand
      (
        response,
        state,
        {namespaceURI: $.pincette.webdav.dav, localName: 'unbind'}
      );
    }



    function sharingWriteCommand(response, state)
    {
      sharingCommand
      (
        response,
        state,
        {namespaceURI: $.pincette.webdav.dav, localName: 'write'}
      );
    }



    function sharingWriteDocumentsCommand(response, state)
    {
      sharingCommand
      (
        response,
        state,
        {namespaceURI: $.pincette.webdav.re, localName: 'write-documents'}
      );
    }



    function showBindings(response, state)
    {
      state.dav.getAllBindings
      (
        response.href,
        function(hrefs)
        {
          var message = '';

          for (var i = 0; i < hrefs.length; ++i)
          {
            message += '<p>' + $.pincette.util.url.getPath(hrefs[i]) + '</p>';
          }

          showLinkBoxAction(message, state);
        }
      );
    }



    function showDetails(documentDetails, position)
    {
      var close =
        $.pincette.util.ui.modal
        (
          documentDetails,
          {
            onShow:
              function(container, overlay, box)
              {
                container.click(function(){close();});
                overlay.click(function(){close();});
                overlay.show();
                container.show();
              }
          }
        );
    }



    function showDetailsAction(state, response)
    {
      assignToState(state, 'showDetailsAction', this).click
      (
        function(e)
        {
          e.stopPropagation();

          if (state.details.length > 0)
          {
            var details = state.details.clone();

            if (state.options.showDetails)
            {
              state.options.showDetails(response, details, $(this), state);
            }
            else
            {
              $.pincette.webdav.ui.
                replaceData(details, response, state.options.fields, state.dav);
              showDetails(details, $(this).offset());
            }
          }
        }
      );
    }



    function showEvents(events, state, from, to, reload)
    {
      var copy = state.events.clone();
      var template = $('.template', copy);

      setReloadActivity(copy, state, from, to, reload);

      for (var i = 0; i < events.length; ++i)
      {
        var entry =
          template.clone().removeClass('template').appendTo(template.parent());

        if (events[i].user)
        {
          $('.user', entry).append
          (
            events[i].user.name ?
              events[i].user.name :
              decodeURIComponent
              (
                $.pincette.util.getLastPathSegment(events[i].user.href)
              )
          );
        }

        $('.occurrence', entry).
          append($.pincette.util.displayDateTime(events[i].occurrence));
        $('.path', entry).append($.pincette.util.addWrapPoints(events[i].path));
        $('.operation', entry).hide();

        $(
          '.operation.' +
            $.pincette.util.fromCamelCase(events[i].operation, '-'),
          entry
        ).show();
      }

      return $.pincette.util.ui.modalWithClose(copy);
    }



    function showFolderExpansion(url, tree, state)
    {
      var copy = state.expand.clone();
      var template = $('.template', copy);

      if (template.length == 0)
      {
        return;
      }

      template.hide();

      addFolderExpansionEntries
      (
        url,
        tree,
        0,
        template,
        $.pincette.util.ui.modalWithClose(copy),
        state
      );
    }



    function showFormAction(state)
    {
      assignToState(state, 'showFormAction', this).
        click(function(){toggleForm(state);});
    }



    function showGroup(group, state)
    {
      documentItems(state).each
      (
        function()
        {
          var value =
            $(this).data('pincette-response').meta[state.options.groupBy];

          if (!group || (value && value[0] == group))
          {
            $(this).show();
          }
          else
          {
            $(this).hide();
          }
        }
      );
    }



    function showGroups(response, state)
    {
      var href =
        $.pincette.util.url.resolveUrl(state.options.rootUrl, '/groups/');
      var tree = {};
      var username =
        decodeURIComponent($.pincette.util.getLastPathSegment(response.href));

      state.dav.search
      (
        href,
        [{field: 'displayname', op: '=', value: username}],
        [{namespaceURI: $.pincette.webdav.dav, localName: 'displayname'}],
        function(response)
        {
          var segments =
            $.pincette.util.
              getPathSegments($.pincette.util.url.getPath(response.href));

          if (segments.length == 2 && segments[0] == 'groups')
          {
            var entry = {};

            entry.path = encodeURIComponent(segments[1]) + '/';
            tree[segments[1]] = entry;
          }
        },
        null,
        function(){showFolderExpansion(href, tree, state);}
      );
    }



    function showLink(response, state)
    {
      showLinkBoxAction
      (
        $.pincette.util.url.resolveUrl
        (
          $.pincette.util.url.getRootUrl(location.href),
          response.stableHref ? response.stableHref : response.href
        ),
        state
      );
    }



    function showLinkBoxAction(message, state)
    {
      $.pincette.util.ui.
        messageBox(message, null, null, state.messageBox.clone());
    }



    function showSearchUrl(state)
    {
      var parameters = 
        $.pincette.util.url.getParameters(location.href);

      for (var name in parameters)
      {
        if (name != 'path' && name != 'lang')
        {
          delete parameters[name];
        }
      }

      var url = $.pincette.util.url.setParameters(location.href, parameters);

      $('input[name="contains"]', state.simpleSearch).
        add('input', state.advancedSearch).each
        (
          function()
          {
            if (isValueSet($(this)))
            {
              url +=
                (url.indexOf('?') != -1 ? '&' : '?') +
                  encodeURIComponent($(this).attr('name')) + '=' +
                  encodeURIComponent($(this).val().trim());
            }
          }
        );

      showLinkBoxAction(url, state);
    }



    function showSearchUrlAction(state)
    {
      assignToState(state, 'showSearchUrlAction', this).
        click(function(){showSearchUrl(state);});
    }



    function showStatistics(events, map, state, from, to, reload)
    {
      var copy = state.statistics.clone();
      var template = $('.template', copy);

      setReloadActivity(copy, state, from, to, reload);
      setChart(copy, events, map, state);

      for (var i = 0; i < events.length; ++i)
      {
        var entry =
          template.clone().removeClass('template').appendTo(template.parent());

        $('.x-axis', entry).addClass('pincette-is-' + map.x.type).
          append(displayEventField(events[i], map.x.field, map.x.type));
        $('.y-axis', entry).addClass('pincette-is-' + map.y.type).
          append(displayEventField(events[i], map.y.field, map.y.type));
      }

      return $.pincette.util.ui.modalWithClose(copy);
    }



    function showVersions(response, state)
    {
      var responses = [];
      var versions = state.versions.clone();

      state.dav.getVersions
      (
        response.href,
        [
          {
            namespaceURI: $.pincette.webdav.dav,
            localName: 'creator-displayname'
          },
          {namespaceURI: $.pincette.webdav.dav, localName: 'comment'},
          {namespaceURI: $.pincette.webdav.dav, localName: 'getlastmodified'},
          {namespaceURI: $.pincette.webdav.dav, localName: 'resourcetype'}
        ],
        function(response){responses.push(response);},
        null,
        function()
        {
          sortVersions(responses);

          displayResultSet
          (
            responses,
            versions,
            state.options.rootUrl,
            !response.isCollection ? state.options.openDocument : null,
            state,
            null,
            false
          );

          $.pincette.util.ui.modalWithClose(versions);
        }
      );
    }



    function showWorkflowOverview(response, state)
    {
      function display()
      {
        displayResultSet
        (
          responses,
          workflowOverview,
          state.options.rootUrl,
          state.options.openDocument,
          state,
          null,
          false
        );
      }

      var responses = [];
      var workflowOverview = state.workflowOverview.clone();

      state.dav.getVersions
      (
        response.href,
        [
          {
            namespaceURI: $.pincette.webdav.dav,
            localName: 'creator-displayname'
          },
          {namespaceURI: $.pincette.webdav.re, localName: 'meta'},
          {namespaceURI: $.pincette.webdav.dav, localName: 'getlastmodified'},
          {namespaceURI: $.pincette.webdav.dav, localName: 'label-name-set'},
          {namespaceURI: $.pincette.webdav.dav, localName: 'resourcetype'}
        ],
        function(response){responses.push(response);},
        null,
        function()
        {
          responses = selectWorkflowVersions(responses);
          sortWorkflowOverview(responses);

          var url = $.pincette.webdav.getWorkflow(response);

          if (url)
          {
            state.dav.getWorkflow
            (
              url,
              function(steps)
              {
                addWorkflowDecisions(responses, steps);
                display();
              }
            );
          }
          else
          {
            display();
          }

          $.pincette.util.ui.modalWithClose(workflowOverview);
        }
      );
    }



    function simpleSearch(state)
    {
      assignToState(state, 'simpleSearch', this);
      setPincetteFields(state.simpleSearch);
    }



    function signOut(state)
    {
      assignToState(state, 'signOut', this).click
      (
        function(e)
        {
          var href = $(this).attr('href');

          if (href)
          {
            e.preventDefault();

            $.pincette.util.ui.confirm
            (
              function(){location.href = href;},
              null,
              state.confirmationBox.clone()
            );
          }
        }
      );
    }



    function sort(state)
    {
      assignToState(state, 'sort', this);
    }



    function sortAction(state)
    {
      assignToState(state, 'sortAction', this).click
      (
        function()
        {
          if
          (
            !state.sort				||
            state.sort.length == 0		||
            documentItems(state).length < 2
          )
          {
            return;
          }

          var copy = state.sort.clone();

          $('.pincette-sort-criterion', copy).
            click
            (
              function()
              {
                $(this).siblings().removeClass('selected');
                $(this).addClass('selected');
                closeDialog(true);
              }
            );

          var closeDialog =
            $.pincette.util.ui.dialogBox
            (
              null,
              {
                onOK:
                  function()
                  {
                    var field =
                      getSortField
                      (
                        $('.pincette-sort-criterion.selected', copy)
                      );

                    if (field)
                    {
                      resortDocuments([field], state);
                      state.lastSortField = field;
                    }

                    return true;
                  }
              },
              copy
            );
        }
      );
    }



    function sortPrincipalElementCriterion(e1, e2)
    {
      var d1 = $(e1).data('pincette-principal');
      var d2 = $(e2).data('pincette-principal');
      var result =
        d1 && !d2 ?
          -1 :
          (
            !d1 && d2 ?
              1 :
              (
                !d1 && !d2 ?
                  0 :
                  (
                    d1.getDisplayname().
                      toLocaleLowerCase().
                      localeCompare(d2.getDisplayname().toLocaleLowerCase())
                  )
              )
          );

      return result;
    }



    function sortResponsesCriterion(r1, r2, sort, state)
    {
      for (var i = 0; i < sort.length; ++i)
      {
        var direction = 1;
        var field = sort[i];

        if (field.indexOf('!') == 0)
        {
          field = field.substring(1);
          direction = -1;
        }

        var result =
          (r1.meta && r1.meta[field]) || (r2.meta && r2.meta[field]) ?
            compareMetaData(r1, r2, field, state) :
            (
              r1[field] && !r2[field] ?
                -1 :
               (
	         !r1[field] && r2[field] ?
		   1 :
                   (
	             !r1[field] && !r2[field] ?
                       0 :
                       (
                         field == 'score' ?
                           (
                             Math.round(r2.score * 100) -
                               Math.round(r1.score * 100)
                           ) :
                           (
                             state.options.fields[field] &&
                               state.options.fields[field].type == 'number' ?
                               (r1[field] - r2[field]) :
                               (
                                 state.options.fields[field] &&
                                   (
                                     state.options.fields[field].type ==
                                       'date' ||
                                       state.options.fields[field].type ==
                                         'time'
                                   ) ?
                                   (r1[field].getTime() - r2[field].getTime()) :
                                   r1[field].toLocaleLowerCase().localeCompare
                                   (
                                     r2[field].toLocaleLowerCase()
                                   )
                               )
                           )
                       )
                   )
               )
            );

        if (result != 0)
        {
          return result * direction;
        }
      }

      return 0;
    }



    function sortResponses(responses, sort, state)
    {
      responses.sort
      (
        function(r1, r2)
        {
          return sortResponsesCriterion(r1, r2, sort, state);
        }
      );
    }



    function sortVersions(responses)
    {
      responses.sort
      (
        function(r1, r2)
        {
          return r2.getlastmodified.valueOf() - r1.getlastmodified.valueOf();
        }
      );
    }



    function sortWorkflowOverview(responses)
    {
      responses.sort
      (
        function(r1, r2)
        {
          return r1.getlastmodified.valueOf() - r2.getlastmodified.valueOf();
        }
      );
    }



    function statistics(state)
    {
      assignToState(state, 'statistics', this);
    }



    function stepWorkflow(response, direction, specific, state)
    {
      $.pincette.util.ui.dialogBox
      (
        null,
        {
          onOK:
            function(element, userClose, reEnableDialogBox)
            {
              stepWorkflowUpdate
              (
                response,
                direction,
                specific,
                isValueSet($('*[name="re_dueDate"]', element)) ?
                  $.pincette.util.timestampISO8601
                  (
                    $.pincette.util.parseDate
                    (
                      $.pincette.util.ui.
                        getFieldValue($('*[name="re_dueDate"]', element))
                    )
                  ) : null,
                isValueSet($('*[name="re_comment"]', element)) ?
                  $.pincette.util.ui.
                    getFieldValue($('*[name="re_comment"]', element)) :
                  null,
                state,
                userClose,
                reEnableDialogBox
              );
            },
          onShow:
            function(element)
            {
              // Delay until here, because we need the display types in
              // order to detect rows.
              $('input,textarea,select', element).
                each(function(){setMetaData($(this), response, state);});

              // We only need the type initialization not the values because
              // the values of this kind of meta-data are set per workflow step.
              $('input,textarea,select', element).val('');
            },
          validate:
            function(element){return validateStepWorkflow(element, state);}
        },
        state.workflowStep.clone()
      );
    }



    function stepWorkflowUpdate
    (
      response,
      direction,
      specific,
      dueDate,
      comment,
      state,
      userClose,
      reEnableDialogBox
    )
    {
      function step()
      {
        var properties = {};

        properties.meta = response.meta;
        properties.meta.re_dueDate = dueDate ? [dueDate] : [];
        properties.meta.re_comment = comment ? [comment] : [];

        if (specific)
        {
          properties.meta.re_workflowStepId = [specific.id];
          properties.meta.re_assignedUser = [specific.assignedUser];
          properties.meta.re_status = [specific.status];
        }
        else
        {
          properties.meta.re_status = [direction];
        }

        var messages = '';

        state.dav.proppatch
        (
          response.href,
          properties,
          null,
          function(response)
          {
            if (response.errors)
            {
              messages += reportProppatchErrors(response.errors);
            }
          },
          function(href, status, method, error, description)
          {
            messages +=
              reportMultistatus(href, status, method, error, description);
          },
          function(status, text, method, headers)
          {
            reEnableDialogBox();
            state.dav.getDefaultFailure()(status, text, method, headers);
            state.dav.uncheckOut(response.href);
          },
          function()
          {
            if (messages.length > 0)
            {
              $.pincette.util.ui.messageBox
              (
                messages,
                null,
                null,
                state.messageBox.clone()
              );

              reEnableDialogBox();
              state.dav.uncheckOut(response.href);
            }
            else
            {
              state.dav.checkIn
              (
                response.href,
                null,
                false,
                function()
                {
                  state.dav.setLabel
                  (
                    response.href,
                    'WORKFLOW_' + $.pincette.util.timestampISO8601(new Date()),
                    '0',
                    null,
                    null,
                    function()
                    {
                      refreshSingle(response, state);
                      userClose();
                    }
                  );
                }
              );
            }
          }
        );
      }

      if (response.checkedOut)
      {
        state.dav.checkIn
        (
          response.href,
          null,
          false,
          function(){state.dav.checkOut(response.href, step)}
        );
      }
      else
      {
        state.dav.checkOut
        (
          response.href,
          step,
          function(status, text, method, headers)
          {
            reEnableDialogBox();
            state.dav.getDefaultFailure()(status, text, method, headers);
          }
        );
      }
    }



    function subscribeRSS(response, state)
    {
      window.open(response.href + '?type=application%2frss%2bxml');
    }



    function tasksAction(state)
    {
      assignToState(state, 'tasksAction', this).click
      (
        function()
        {
          state.dav.getCurrentUser
          (
            location.href,
            function(response)
            {
              executeSearch
              (
                state.options.searchUrl,
                [
                  {
                    field: 'meta.re_assignedUser',
                    op: '=',
                    value: response.name
                  }
                ],
                ['re_dueDate', 'name'],
                state
              );
            }
          );
        }
      );
    }



    function toggleForm(state)
    {
      if (state.showFormAction)
      {
        if (state.showFormAction.hasClass('closed'))
        {
          openForm(state);
        }
        else
        {
          closeForm(state);
        }
      }
    }



    function topAction(state)
    {
      assignToState(state, 'topAction', this).click
      (
        function()
        {
          state.history = new Array();
          state.historyPosition = 0;
          loadFolder(state.options.rootUrl, false, state);
        }
      );
    }



    function trash(state)
    {
      function tryRoot()
      {
        state.dav.exists
        (
          state.options.rootUrl + '.Trash/',
          function(response)
          {
            if (response)
            {
              loadFolder(response.href, false, state);
            }
          }
        );
      }

      var result = assignToState(state, 'trash', this);

      result.click
      (
        function()
        {
          state.dav.getHome
          (
            state.options.rootUrl + 'users/me',
            function(home)
            {
              if (home)
              {
                state.dav.exists
                (
                  home +
                    (
                      home.length > 0 && home.charAt(home.length - 1) == '/' ?
                        '' : '/'
                    ) + '.Trash',
                  function(response)
                  {
                    if (response)
                    {
                      loadFolder(response.href, false, state);
                    }
                    else
                    {
                      tryRoot();
                    }
                  }
                );
              }
              else
              {
                tryRoot();
              }
            },
            null,
            tryRoot
          );
        }
      );

      return result;
    }



    function trimValue(value)
    {
      return value ? value.trim() : '';
    }



    function uncheckOut(response, state)
    {
      $.pincette.util.ui.confirm
      (
        function()
        {
          state.dav.uncheckOut
          (
            response.href,
            function(){refreshSingle(response, state);}
          );
        },
        null,
        state.confirmationBox.clone()
      );
    }



    function uniqueVisitors(response, state)
    {
      getStatistics
      (
        response,
        {
          count: true,
          operations: ['get', 'update'],
          groupBy: ['day', 'peer'],
          orderBy: ['day', 'peer']
        },
        {
          x: {field: 'day', type: 'date'},
          y: {field: 'count', type: 'integer'}
        },
        state,
        function(events)
        {
          var currentEvent;
          var result = new Array();

          for (var i = 0; i < events.length; ++i)
          {
            if (!currentEvent)
            {
              currentEvent = {day: events[i].day, count: 1};
            }
            else
            {
              if (currentEvent.day.getTime() != events[i].day.getTime())
              {
                result.push(currentEvent);
                currentEvent = {day: events[i].day, count: 1};
              }
              else
              {
                currentEvent.count += 1;
              }
            }
          }

          if (currentEvent)
          {
            result.push(currentEvent);
          }

          return result;
        }
      );
    }



    function unlock(response, state)
    {
      state.dav.unlock
      (
        response.href,
        response.locks[0].token,
        function(){refreshSingle(response, state);},
        function()
        {
          $.pincette.util.ui.messageBox
          (
            getResource("err-locked") +
              (response.locks[0].owner ? response.locks[0].owner : '?'),
            null,
            null,
            state.messageBox.clone()
          );
        }
      );
    }



    function updateMetaData
    (
      element,
      response,
      lockToken,
      state,
      done,
      reEnableDialogBox
    )
    {
      function end()
      {
        if (done)
        {
          done();
        }

        refreshSingle(response, state);
      }

      var controls =
        $('input,textarea,select', element).
          filter(function(){return isValueSet($(this));});
      var properties = {};

      properties.meta = response.meta;

      // Clear meta-data that will be edited.
      $('input,textarea,select', element).
        each(function(){delete properties.meta[$(this).attr('name')];});

      controls.each
      (
        function()
        {
          var name = $(this).attr('name');

          if (!properties.meta[name])
          {
            properties.meta[name] = new Array();
          }

          properties.meta[name].
            push(getValue(name, $(this).val().trim(), state));
        }
      );

      if (state.options.missingMetaDataProperty && notSetMetaData(element))
      {
        properties.meta[state.options.missingMetaDataProperty] = ["true"];
      }

      var messages = '';

      state.dav.proppatch
      (
        response.href,
        properties,
        lockToken,
        function(response)
        {
          if (response.errors)
          {
            messages += reportProppatchErrors(response.errors);
          }
        },
        function(href, status, method, error, description)
        {
          messages +=
            reportMultistatus(href, status, method, error, description);
        },
        function(status, text, method, headers)
        {
          reEnableDialogBox();
          state.dav.getDefaultFailure()(status, text, method, headers);
        },
        function()
        {
          if (messages.length > 0)
          {
            $.pincette.util.ui.messageBox
            (
              messages,
              null,
              null,
              state.messageBox.clone()
            );

            reEnableDialogBox();
          }
          else
          {
            if (lockToken)
            {
              state.dav.unlock(response.href, lockToken, end);
            }
            else
            {
              end();
            }
          }
        }
      );
    }



    function updateRelations
    (
      element,
      response,
      relationName,
      notEdited,
      lockToken,
      state,
      done,
      reEnableDialogBox
    )
    {
      function afterAll()
      {
        if (done)
        {
          done();
        }

        refreshSingle(response, state);
      }

      var messages = '';
      var relations = notEdited;

      $('.url', element).not('.place-holder').
        each
        (
          function()
          {
            relations.
              push({name: relationName, href: $(this).data('pincette-url')});
          }
        );

      if (relationsChanged(response.referringTo, relations))
      {
        state.dav.proppatch
        (
          response.href,
          {referringTo: relations},
          lockToken,
          function(response)
          {
            if (response.errors)
            {
              messages += reportProppatchErrors(response.errors);
            }
          },
          function(href, status, method, error, description)
          {
            messages +=
              reportMultistatus(href, status, method, error, description);
          },
          function(status, text, method, headers)
          {
            reEnableDialogBox();
            state.dav.getDefaultFailure()(status, text, method, headers);
          },
          function()
          {
            if (messages.length > 0)
            {
              $.pincette.util.ui.messageBox
              (
                messages,
                null,
                null,
                state.messageBox.clone()
              );

              reEnableDialogBox();
            }
            else
            {
              if (lockToken)
              {
                state.dav.unlock(response.href, lockToken, afterAll);
              }
              else
              {
                afterAll();
              }
            }
          }
        );
      }
      else
      {
        if (lockToken)
        {
          state.dav.unlock(response.href, lockToken, afterAll);
        }
        else
        {
          afterAll();
        }
      }
    }



    function updateUser(url, element, current, state, done, reEnableDialogBox)
    {
      var properties = {};

      if
      (
        current.displayname					||
        isValueSet($('input[name="displayname"]', element))
      )
      {
        properties.displayname =
          trimValue($('input[name="displayname"]', element).val());
      }

      if
      (
        current.backup					||
        isValueSet($('select[name="backup"]', element))
      )
      {
        properties.re_backup = $('select[name="backup"]', element).val();
      }

      if
      (
        current.email					||
        isValueSet($('input[name="email"]', element))
      )
      {
        properties.re_email =
          trimValue($('input[name="email"]', element).val());
      }

      if
      (
        current.home					||
        isValueSet($('input[name="home"]', element))
      )
      {
        properties.re_home = trimValue($('input[name="home"]', element).val());
      }

      if
      (
        current.language					||
        isValueSet($('select[name="language"]', element))
      )
      {
        properties.re_language =
          trimValue($('select[name="language"]', element).val());
      }

      if (isValueSet($('input[name="password"]', element)))
      {
        properties.re_password =
          $('input[name="password"]', element).val().trim();
      }

      if
      (
        properties.re_password				||
        current.displayname != properties.displayname	||
        current.backup != properties.re_backup		||
        current.email != properties.re_email		||
        current.home != properties.re_home		||
        current.language != properties.re_language
      )
      {
        var messages = '';

        state.dav.proppatch
        (
          url,
          properties,
          null,
          function(response)
          {
            if (response.errors)
            {
              messages += reportProppatchErrors(response.errors);
            }
          },
          function(href, status, method, error, description)
          {
            messages +=
              reportMultistatus(href, status, method, error, description);
          },
          function(status, text, method, headers)
          {
            reEnableDialogBox();
            state.dav.getDefaultFailure()(status, text, method, headers);
          },
          function()
          {
            if (messages.length > 0)
            {
              $.pincette.util.ui.messageBox
              (
                messages,
                null,
                null,
                state.messageBox.clone()
              );

              reEnableDialogBox();
            }
            else
            {
              if (done)
              {
                done(url, properties);
              }
            }
          }
        );
      }
      else
      {
        if (done)
        {
          done(url, properties);
        }
      }
    }



    function updateWorkflow
    (
      element,
      response,
      displayNamesToNames,
      lockToken,
      state,
      done,
      reEnableDialogBox
    )
    {
      var count = 0;
      var result = '<?xml version="1.0" encoding="UTF-8"?><workflow>';

      $('input[name="user"]', element).
        filter(function(){return isValueSet($(this));}).each
        (
          function()
          {
            var user =
              displayNamesToNames[$.pincette.util.ui.getFieldValue($(this))];

            if (user)
            {
              var row = $.pincette.util.getClosestRow($(this));

              $('select[name="status"]', row).
                filter(function(){return isValueSet($(this));}).each
                (
                  function()
                  {
                    ++count;

                    
                    result +=
                      '<step' +
                        (
                          $(row).attr('id') ?
                            (' id="' + $(row).attr('id') + '"') : ''
                        ) + '><assigned-user>' +
                        $.pincette.util.escapeXMLText(user) +
                        '</assigned-user><status>' +
                        $.pincette.util.escapeXMLText
                        (
                          $.pincette.util.ui.getFieldValue($(this))
                        ) + '</status></step>';
                  }
                );
            }
          }
        );

      if (count > 1)
      {
        result += '</workflow>';

        state.dav.put
        (
          response.href,
          result,
          {'Content-Type': 'application/vnd.pincette.workflow+xml'},
          lockToken,
          function()
          {
            state.dav.unlock
            (
              response.href,
              lockToken,
              function()
              {
                if (done)
                {
                  done();
                }

                refreshSingle(response, state);
              }
            );
          },
          function(status, text, method, headers)
          {
            reEnableDialogBox();
            state.dav.getDefaultFailure()(status, text, method, headers);
          }
        );
      }
    }



    function upload(state)
    {
      assignToState(state, 'upload', this);
    }



    function uploadAction(response, state)
    {
      uploadFiles(response, false, state);
    }



    function uploadFiles(response, expandZip, state)
    {
      var box = state.upload.clone();
      var entries = {};
      var uploaded = 0;
      var uploading = 0;
      var input = $('input[name="files[]"]', box);

      input.attr('multiple', response.isCollection ? 'true' : 'false');

      var options =
        {
          add:
            function(e, data)
            {
              if
              (
                expandZip						&&
                !$.pincette.util.endsWith(data.files[0].name, '.zip')
              )
              {
                $.pincette.util.ui.messageBox
                (
                  getResource("err-zip"),
                  null,
                  null,
                  state.messageBox.clone()
                );

                return;
              }

              if ((uploading > 0 || expandZip) && !response.isCollection)
              {
                return;
              }

              var entry =
                $(
                  '<div class="pincette-upload-entry">' +
                    '<div class="filename"><![CDATA[' + data.files[0].name +
                    ']]></div>' +
                    '<div class="progress">' +
                      '<div class="progress-bar">' +
                        '<div class="bar"><div/></div>' +
                      '</div>' +
                      '<div class="control">' +
                        '<div><span class="cancel"/></div>' +
                      '</div>' +
                    '</div>' +
                  '</div>'
                );

              $('.progress .bar > div', entry).width(0);
              entries[data.files[0].name] = entry;
              $('.files', box).append(entry);

              data.realUrl =
                response.isCollection ?
                  (
                    response.href +
                      (expandZip ? '' : encodeURIComponent(data.files[0].name))
                  ) : response.href;
              data.url =
                state.options.uploadUrl + '?url=' + data.realUrl +
                  (
                    $('input[name^="type:"]', box).prop('checked') ?
                      (
                        '&type=' +
                          $('input[name^="type:"]', box).attr('name').
                            substring(5)
                      ) : ''
                  );

              ++uploading;

              var jqXHR = data.submit();

              $('.progress .control .cancel', entry).
                click(function(){jqXHR.abort();});
            },
          multipart: true,
          sequentialUploads: true,
          singleFileUploads: true,
          type: 'POST',
          xhrFields: {withCredentials: true}
        }

      if (state.dav.oauth)
      {
        options.beforeSend =
          function(jqXHR, settings)
          {
            if (state.dav.oauth)
            {
              state.dav.oauth.setAccessToken(jqXHR, settings);
            }
          };
      }

      if (state.options.parameters.onbehalfof)
      {
        options.headers =
          {'X-be.re.On-Behalf-Of': state.options.parameters.onbehalfof[0]};
      }

      input.fileupload(options);

      input.bind
      (
        'fileuploaddone',
        function(e, data)
        {
          --uploading;
          ++uploaded;
          $('.progress .control span.cancel', entries[data.files[0].name]).
            removeClass('cancel').addClass('done');
          $('.progress .bar div', entries[data.files[0].name]).width('100%');

          window.setTimeout
          (
            function(){entries[data.files[0].name].remove();},
            1000
          );

          if (state.options.afterUpload)
          {
            state.options.afterUpload(data.realUrl, state);
          }
        }
      );

      input.bind
      (
        'fileuploadfail',
        function(e, data)
        {
          --uploading;
          $('.progress .bar > div', entries[data.files[0].name]).width(0);

          var span =
            $('.progress .control span.cancel', entries[data.files[0].name]);

          if (data.jqXHR.status != 0)
          {
            span.removeClass('cancel').addClass('error');

            span.click
            (
              function()
              {
                $.pincette.util.ui.messageBox
                (
                  $.pincette.webdav.ui.getStatusMessage
                  (
                    data.jqXHR.status,
                    'PUT',
                    data.jqXHR.responseText
                  ),
                  null,
                  null,
                  state.messageBox.clone()
                );
              }
            );
          }
          else
          {
            span.removeClass('cancel').addClass('done'); // Aborted by user.
          }
        }
      );

      input.bind
      (
        'fileuploadprogress',
        function(e, data)
        {
          var bar = $('.progress .bar', entries[data.files[0].name]);

          $('div', bar).width
          (
            data.loaded == data.total ? // Otherwise we have rounding errors.
              '100%' : (bar.width() * data.loaded / data.total)
          );
        }
      );

      var closeModal;

      function closeModalIfDone()
      {
        if (uploaded > 0)
        {
          if (response.isCollection)
          {
            if (response.href == getOpenFolder(state))
            {
              refresh(response.href, state);
            }
          }
          else
          {
            refreshSingle(response, state);
          }
        }
      }

      closeModal =
        $.pincette.util.ui.modal
        (
          box,
          {
            close:
              function()
              {
                if (uploading == 0)
                {
                  closeModal();
                }
              },
            onClose:
              function(container, overlay, box, close)
              {
                close();
                closeModalIfDone();
              },
            onShow:
              function(container, overlay, box)
              {
                function addClose()
                {
                  if ($('.pincette-title-bar .close', box).length == 0)
                  {
                    $.pincette.util.ui.addClose(box, closeModalIfDone);
                  }
                }

                overlay.show();
                container.show();
                addClose();
              }
          }
        );
    }



    function uploadsPerDay(response, state)
    {
      getStatistics
      (
        response,
        {
          count: true,
          operations: ['update'],
          groupBy: ['day'],
          orderBy: ['day']
        },
        {
          x: {field: 'day', type: 'date'},
          y: {field: 'count', type: 'integer'}
        },
        state
      );
    }



    function user(state)
    {
      assignToState(state, 'user', this);
    }



    function userMetaData(state)
    {
      assignToState(state, 'userMetaData', this);
    }



    function validate(element, state)
    {
      return $('.pincette-bad-value', element).length == 0;
    }



    function validateEditUser(element, state)
    {
      var result =
        validateEmail(element, state) && comparePasswords(element, state);

      return result;
    }



    function validateEmail(element, state)
    {
      if
      (
        isValueSet($('input[name="email"]', element))			&&
        !$.pincette.util.
          isEmail($('input[name="email"]', element).val().trim())
      )
      {
        $.pincette.util.ui.messageBox
        (
          getResource("err-email"),
          null,
          null,
          state.messageBox.clone()
        );

        return false;
      }

      return true;
    }



    function validateMetaData(element, state)
    {
      $('input', element).
        filter(function(){return isValueSet($(this));}).
        filter
        (
          function()
          {
            var field = state.options.fields[$(this).attr('name')];

            return field && field.values;
          }
        ).
        each
        (
          function()
          {
            var found = false;
            var value = $(this).val();
            var values =
              selectMatchingValues
              (
                state.options.fields[$(this).attr('name')].values,
                value
              );

            for (var i = 0; !found && i < values.length; ++i)
            {
              if (values[i] == value)
              {
                found = true;
              }
            }

            if (!found)
            {
              $(this).addClass('pincette-bad-value');
            }
          }
        );

      return $('input.pincette-bad-value', element).length == 0;
    }



    function validateNewUser(element, state)
    {
      if
      (
        !isValueSet($('input[name="name"]', element))		||
        !isValueSet($('input[name="password"]', element))	||
        !isValueSet($('input[name="again"]', element))
      )
      {
        $.pincette.util.ui.messageBox
        (
          getResource("err-missing-fields"),
          null,
          null,
          state.messageBox.clone()
        );

        return false;
      }

      return validateEmail(element, state) && comparePasswords(element, state);
    }



    function validateStepWorkflow(element, state)
    {
      var result = true;

      $('input[name="re_dueDate"]', element).each
      (
        function()
        {
          $(this).removeClass('pincette-bad-value');

          if (isValueSet($(this)))
          {
            var value = $.pincette.util.ui.getFieldValue($(this));

            if (!$.pincette.util.isDate(value))
            {
              $(this).addClass('pincette-bad-value');
              result = false;
            }
            else
            {
              var date = $.pincette.util.parseDate(value);
              var now = new Date();

              if ($.pincette.util.isBeforeDate(date, now))
              {
                $(this).addClass('pincette-bad-value');
                result = false;
              }
            }
          }
        }
      );

      return result;
    }



    function versions(state)
    {
      assignToState(state, 'versions', this);
    }



    function workflow(state)
    {
      assignToState(state, 'workflow', this);
    }



    function workflowOverview(state)
    {
      assignToState(state, 'workflowOverview', this);
    }



    function workflowStep(state)
    {
      assignToState(state, 'workflowStep', this);
    }



    function workflowSteps(state)
    {
      assignToState(state, 'workflowSteps', this);
    }



    var initMethods =
      {
        advancedSearch: {method: advancedSearch, init: true},
        advancedSearchAction: {method: advancedSearchAction, init: true},
        backAction: {method: backAction, init: true},
        boxMenu: {method: boxMenu, init: true},
        button: {method: button, init: true},
        cancelAction: {method: cancelAction, init: true},
        chart: {method: chart, init: true},
        checkIn: {method: checkIn, init: true},
        clearAction: {method: clearAction, init: true},
        comment: {method: comment, init: true},
        confirmationBox: {method: confirmationBox, init: true},
        count: {method: count, init: true},
        dateBox: {method: dateBox, init: true},
        details: {method: details, init: true},
        documents: {method: documents, init: true},
        events: {method: events, init: true},
        expand: {method: expand, init: true},
        folders: {method: folders, init: true},
        forms: {method: forms, init: true},
        forwardAction: {method: forwardAction, init: true},
        helpAction: {method: helpAction, init: true},
        init: {method: init, init: false},
        loading: {method: loading, init: true},
        meAction: {method: meAction, init: true},
        messageBox: {method: messageBox, init: true},
        metaData: {method: metaData, init: true},
        metaDataAddValue: {method: metaDataAddValue, init: true},
        nameBox: {method: nameBox, init: true},
        navigation: {method: navigation, init: true},
        newDocument: {method: newDocument, init: true},
        newUser: {method: newUser, init: true},
	path: {method: path, init: true},
        periodBox: {method: periodBox, init: true},
	previewBox: {method: previewBox, init: true},
	relations: {method: relations, init: true},
	reporter: {method: reporter, init: true},
	searchAction:
          {method: searchAction, init: true, depends: ['simpleSearch']},
        searchDoneAction: {method: searchDoneAction, init: true},
        selectPrincipals: {method: selectPrincipals, init: true},
        sharing: {method: sharing, init: true},
        showDetailsAction: {method: showDetailsAction, init: false},
        showFormAction: {method: showFormAction, init: true},
        showSearchUrlAction: {method: showSearchUrlAction, init: true},
        simpleSearch: {method: simpleSearch, init: true},
        signOut: {method: signOut, init: true},
        sort: {method: sort, init: true},
        sortAction: {method: sortAction, init: true},
        statistics: {method: statistics, init: true},
        tasksAction: {method: tasksAction, init: true},
        topAction: {method: topAction, init: true},
        trash: {method: trash, init: true},
        upload: {method: upload, init: true},
        user: {method: user, init: true},
        userMetaData: {method: userMetaData, init: true},
        versions: {method: versions, init: true},
        workflow: {method: workflow, init: true},
        workflowOverview: {method: workflowOverview, init: true},
        workflowStep: {method: workflowStep, init: true},
        workflowSteps: {method: workflowSteps, init: true}
      };
    var toolMethods = {pickUrl: pickUrl};

    $.fn.pincette = function(method, options, state)
    {
      var index;
      var m;

      if (typeof method === 'object' || !method)
      {
        m = 'init';
        index = method === 'object' ? 0 : 1;
      }
      else
      {
        m = method;
        index = 1;
      }

      if (initMethods[m])
      {
        var data = this.data('pincette-' + method);

        if (!data)
        {
          var allOptions = prepareOptions(arguments[index]);

          data = arguments[index + 1] ? arguments[index + 1] : new State();
          this.data('pincette-' + method, data);
          prepareState(data, allOptions);

          var args =
            arguments.length > index + 2 ?
              [data].concat(Array.prototype.slice.call(arguments, index + 2)) :
              [data];

          return initMethods[m].method.apply(this, args);
        }
      }
      else
      {
        if (toolMethods[m])
        {
          var args = Array.prototype.slice.call(arguments, 1);

          return this.each(function(){toolMethods[m].apply(this, args);});
        }

        $.error('Method ' + m + ' does not exist on jQuery.pincette.');
      }
    }
  }
)(jQuery);
